/***************************************************************************
*                          aclCheck.cpp  -  description
*                             -------------------
*    begin                : Sam Feb  1 17:08:52 CET 2003
*    copyright            : (C) 2003 by the gomd group at savannah.org/
*    email                : rpons@rinu.org, brian@axehind.com, jp80@libero.it, mosixview@t-online.de
****************************************************************************/

/**
 * NOTES:
 * Last update by rejected
 *
 * Contains code to handle Access Control Lists check
 */

#include "aclCheck.h"

/**
 * constructor for class aclCheck
 * @param aclMap is a valid aclRulebook object filled with ACLs
 * @param localIP contains the IP of local node
 */
aclCheck::aclCheck(multimap<string,string> aclMap, string localIP)
{
    m_aclMap = aclMap;
    m_localIP = localIP;

}

/**
 * returns the security level associated to a given IP address
 * NOTE: uses the "last-found" mechanism: if the map contains two (or more)
 *       rules for a single IP, the last one will be returned
 * @param ipToCheck is a string including the IP to be searched into the ACLs map
 * @return the seclevel associated to an IP address (if found into the map); otherwise
 *  return ACL_DENY_CONTROL (for security reasons).
 */
acl_seclevel_t aclCheck::getAssociatedSecurityLevel(string ipToCheck)
{

    acl_seclevel_t retLevel = ACL_DENY_CONTROL; //return value (set by dfeault to "deny all"
    string ipFromMap; //tmp var
    unsigned int counter = 1;
    string subnetOfIpFromMap; //subnet of ipFromMap
    string subnetOfIpToCheck; //subnet of the given IP
    multimap<string,string>::iterator m_iter; //iterator for the (multi)map of ACLs

    subnetOfIpToCheck = m_util.getSubnetFromIP(ipToCheck);

    for(m_iter = m_aclMap.begin(); m_iter != m_aclMap.end(); ++m_iter) //for each entry in the ACLs map
    {
        ipFromMap = m_iter->first; //get the ip in the map

        //REMOVE_FOR_DEBUG cout << "   --- ACL rule #" << counter << " verification ---" << endl;
        //REMOVE_FOR_DEBUG cout << "   ipFromMap before macro expansion:  #" << ipFromMap << "#" << endl;

        if(ipFromMap=="LOCAL_IP") //macro expansion, if necessary...
        {
            ipFromMap=m_localIP; //now we'll use real IP for local machine
        }
        //REMOVE_FOR_DEBUG cout << "   ipFromMap after macro expansion :  #" << ipFromMap << "#" << endl;
        //REMOVE_FOR_DEBUG cout << "   ipToCheck passed                :  #" << ipToCheck << "#" << endl;


        if(ipFromMap=="LOCAL_SUBNET" ) //if the rule specifies any node or a specific IP...
        {

            subnetOfIpFromMap = m_util.getSubnetFromIP(m_localIP); //get the subnet for ipFromMap
            //REMOVE_FOR_DEBUG cout <<  "   subnets to be compared : " << subnetOfIpFromMap << "<->" << subnetOfIpToCheck << endl;

            if(subnetOfIpFromMap==subnetOfIpToCheck)
            {
                //REMOVE_FOR_DEBUG cout <<  "   getAssSecLvl()         :" << m_iter->second << endl;
                return stringToSeclevel(m_iter->second);
                //REMOVE_FOR_DEBUG cout <<  "   stringToSeclevel()     :" << retLevel << endl;
                //REMOVE_FOR_DEBUG cout <<  "   type of rule           : ON_LOCAL_SUBNET" << endl; //rule matches!
                //REMOVE_FOR_DEBUG cout <<  "   response               : rule DO match!" << endl; //rule matches!
            }
            else
            {
                //REMOVE_FOR_DEBUG
                cout <<  "   type of rule           : ON_LOCAL_SUBNET" << endl; //rule matches!
                //REMOVE_FOR_DEBUG
                cout <<  "   response               : rule DOES NOT match!" << endl;
                retLevel = ACL_DENY_CONTROL; //[redundant (but more clear)]
            }
        }
        else if(ipFromMap=="ANY_IP" || ipFromMap==ipToCheck) //if the rule specifies any node or a specific IP...
        {
            //REMOVE_FOR_DEBUG cout <<  "   getAssSecLvl()         :" << m_iter->second << endl;
            return stringToSeclevel(m_iter->second);
            //REMOVE_FOR_DEBUG cout <<  "   stringToSeclevel()     :" << retLevel << endl;
            //REMOVE_FOR_DEBUG cout <<  "   type of rule           : ANY_IP || IP was matched" << endl; //rule matches!
            //REMOVE_FOR_DEBUG cout <<  "   response               : rule DO match!" << endl; //rule matches!
        }
        else //the rule doesn't match!
        {
            //REMOVE_FOR_DEBUG cout <<  "   response               : rule DOES NOT match!" << endl;
            retLevel = ACL_DENY_CONTROL; //[redundant (but more clear)]
        }

        //REMOVE_FOR_DEBUG cout << "   --- ACL rule #" << counter << " verification ---\n" << endl;
        ++counter;
    }
    return retLevel;
}

/**
 * convert a string to its seclevel value
 * @param toBeConverted is the string to be converted
 * @return the associated seclevel (if available,otherwise returns ACL_DENY_CONTROL value)
 */
int aclCheck::stringToSeclevel(string toBeConverted)
{
    if (toBeConverted=="ACL_LOCAL_INFOS_ONLY")
    {
        return ACL_LOCAL_INFOS_ONLY;
    }
    else if (toBeConverted=="ACL_FULL_CONTROL")
    {
        return ACL_FULL_CONTROL;
    }
    return ACL_DENY_CONTROL; //default value is ACL_DENY_CONTROL
}

/**
 * verifies if a client (specified by his IP address)
 * can execute a function (specified by its identifier)
 * @param ipToCheck is the IP of the client requesting a specific function
 * @param functionById is the identifier of the function client want to execute
 * @return a value-based answer for the check-permission request
 *  Available values:
 *   0 => success! (client is allowed and can get the requested function!)
 *  -1 => failure! (client is allowed but cannot get the function because it requires special permissions!)
 *  -2 => failure! (client is _not_ allowed so he can get _no_ function!)
 */
int aclCheck::validatePermissions(string ipToCheck, int functionById)
{
    //print some debug stats
    //REMOVE_FOR_DEBUG cout << "\n   --- Preliminar ACL verification ---" << endl;
    //REMOVE_FOR_DEBUG cout << "   ipTocheck    :" << ipToCheck << endl;
    //REMOVE_FOR_DEBUG cout << "   localIP      :" << m_localIP <<  endl;
    //REMOVE_FOR_DEBUG cout << "   functionById :" << functionById << endl;
    //REMOVE_FOR_DEBUG cout << "   aclMap.size():" << m_aclMap.size() << endl;
    //REMOVE_FOR_DEBUG cout << "   --- Preliminar ACL verification ---\n" << endl;

    //get the security level associated to the given IP address
    acl_seclevel_t lvl = getAssociatedSecurityLevel(ipToCheck);

    //if the IP is blacklisted...
    if(lvl==ACL_DENY_CONTROL)
    {
        return -2; //return the error code!
    }

    //if the IP is allowed but it cannot get the specified function
    if((functionById>=20 && functionById<=22) &&
            (lvl!=ACL_FULL_CONTROL))
    {
        return -1; //return the error code!
    }

    //if we are here => client is allowed and can get the specified function
    return 0; //return the code associated to this positive event
}
