#include "chSupport.h"

/**
 * constructor for class chSupport
 * used to fill the syntactic rules for client requests
 */
chSupport::chSupport()
{
	// ID		       		   CMD         LONG OPT. 	    SHORT OPT.  MNEM	VALIDVALUES
	m_validRules.push_back(requestRule("get",	"quit"  	,	"q",	"0",	NA));
	m_validRules.push_back(requestRule("get",	"id"		,	"i",	"1",	NA));
	m_validRules.push_back(requestRule("get",	"cpus"  	,	"c",	"2",	NODELIST));
	m_validRules.push_back(requestRule("get",	"load"  	,	"l",	"3",	NODELIST));
	m_validRules.push_back(requestRule("get",	"mem"	,		"m",	"4",	NODELIST));
	m_validRules.push_back(requestRule("get",	"rmem"  	,	"r",	"5",	NODELIST));
	m_validRules.push_back(requestRule("get",	"speed"	,		"s",	"6",	NODELIST));
	m_validRules.push_back(requestRule("get",	"status"	,	"a",	"7",	NODELIST));
	m_validRules.push_back(requestRule("get",	"tmem"	,		"t",	"8",	NODELIST));
	m_validRules.push_back(requestRule("get",	"util"	,		"u",	"9",	NODELIST));
	m_validRules.push_back(requestRule("get",	"block"	,		"b",	"10",	NA));
	m_validRules.push_back(requestRule("get",	"ip"	,		"p",	"11",	NA));
	m_validRules.push_back(requestRule("get",	"bypid"	,		"bp",	"12",	NUMBER));
	m_validRules.push_back(requestRule("get",	"nodelist",		"nl",	"13",	NA));
	m_validRules.push_back(requestRule("get",	"deadnodes",		"dn",	"14",	NA));
	m_validRules.push_back(requestRule("set",	"sblock",		"sb",	"20",	NA));
	m_validRules.push_back(requestRule("set",	"ublock",		"ub",	"21",	NA));
	m_validRules.push_back(requestRule("set",	"decay"	,		"d",	"22",	"clear cpujob exec execonce fast inherit iojob own slow"));
	m_validRules.push_back(requestRule("get",	"lastrequest",		"lr",	"30",	NA));
	m_validRules.push_back(requestRule("get",	"allrequests",		"ar",	"31",	NA));
	m_validRules.push_back(requestRule("get",	"uprequest",		"up",	"32",	NA));
	m_validRules.push_back(requestRule("get",	"downrequest"	,	"do",	"33",	NA));
	m_validRules.push_back(requestRule("get",	"snapshot",		"sn",	"34",	NA));
	m_validRules.push_back(requestRule("start",	"autodiscovery",	"ad",	"100",	NUMBER));
	m_validRules.push_back(requestRule("start",	"cscollector",		"cs",	"101",	NA));
	m_validRules.push_back(requestRule("get",	"errorcodes",		"ec",	"102",	NA));
	m_validRules.push_back(requestRule("exec",	"cmd",			"cmd",	"103",	STRING));
	m_validRules.push_back(requestRule("exec",	"con",			"con",	"108",	STRING));
	m_validRules.push_back(requestRule("exec",	"cwc",			"cwc",	"104",	STRING));
	m_validRules.push_back(requestRule("start",	"autodiscovery2",	"ad2",	"105",	NA));
	m_validRules.push_back(requestRule("exec",	"dump",			"dump",	"106", 	NUMBER));
	m_validRules.push_back(requestRule("get",	"temp",			"tm",	"107", 	NA));

	#ifdef __LIBGTOP
	m_validRules.push_back(requestRule("get" ,	"help"	,		"h",	"99", 	"1 2 3 4 5"));
	m_validRules.push_back(requestRule("get",	"cpuload",		"cl",	"40", 	REMOTE_OR_LOCAL_NODE));
	m_validRules.push_back(requestRule("get",	"cpuinfo",		"ci",	"41", 	REMOTE_OR_LOCAL_NODE));
	m_validRules.push_back(requestRule("get",	"netload",		"nl",	"42", 	"lo eth0 eth1eth2 eth3"));
	m_validRules.push_back(requestRule("get",	"pidlist",		"pl",	"43", 	REMOTE_OR_LOCAL_NODE));
	m_validRules.push_back(requestRule("get",	"procstatus",		"ps",	"44", 	REMOTE_OR_LOCAL_NODE));
	m_validRules.push_back(requestRule("get",	"mountlist",		"ml",	"45", 	REMOTE_OR_LOCAL_NODE));
	m_validRules.push_back(requestRule("get",	"mountusage",		"ul",	"46", 	REMOTE_OR_LOCAL_NODE));
	m_validRules.push_back(requestRule("get",	"uptime",		"ut",	"47", 	REMOTE_OR_LOCAL_NODE));
	m_validRules.push_back(requestRule("get",	"meminfo",		"mi",	"48", 	REMOTE_OR_LOCAL_NODE));
	m_validRules.push_back(requestRule("get",	"swapusage",		"su",	"49", 	REMOTE_OR_LOCAL_NODE));
	m_validRules.push_back(requestRule("get",	"loadaverage",		"la",	"50", 	REMOTE_OR_LOCAL_NODE));
	m_validRules.push_back(requestRule("get",	"procstate",		"pa",	"51", 	NUMBER));
	m_validRules.push_back(requestRule("get",	"procuid",		"pu",	"52", 	NUMBER));
	m_validRules.push_back(requestRule("get",	"procmemory",		"pm",	"53", 	NUMBER));
	m_validRules.push_back(requestRule("get",	"proctime",		"pt",	"54", 	NUMBER));
	m_validRules.push_back(requestRule("get",	"procsignal",		"pg",	"55", 	NUMBER));
	#else
	m_validRules.push_back(requestRule("get" ,	"help"	,		"h",	"99", 	"1 2 3 4"));
	#endif
}

/**
 * sends the list of error codes
 * @return a string with the errorc codes
 */
string chSupport::showErrorCodes()
{
	string ret;

	ret =  "DAEMON ERROR CODES:\n\n";
	ret += "Code	Class	Description\n";
	ret += "-10	utils	I/O error occurred\n";
	ret += "-11	utils	conversion error\n";
	ret += "-12	utils	subnet finding error\n";
	ret += "-20	omInfo	oM not started or node not configured\n";
	ret += "-101	utils	default oM error code for configured\n";
	ret += "		but not running node\n";
	return ret;
}

/**
 * send available commands list to the client
 * @param helpPage is the page to be returned
 * @return a string with the requested help page; an error message otherwise
 */
string chSupport::showHelp(string helpPage)
{
	string helpBuffer;

	//help page handling
	if(helpPage == "1" || helpPage == "")
	{
		helpBuffer =  "PROGRAM COMMANDS:\n\n";
		helpBuffer += "Usage:\n";
		helpBuffer += "get OPTION\n\n";
		helpBuffer += "OPTIONS\n";
		helpBuffer += "help/h [1]  -> show these  commands\n";
		helpBuffer += "help/h 2    -> show local getter commands\n";
		helpBuffer += "help/h 3    -> show multiple-nodes getter commands\n";
		helpBuffer += "help/h 4    -> show setter commands\n";
		helpBuffer += "quit/q      -> close connection\n";
	}
	else if(helpPage == "2")
	{
		helpBuffer =  "LOCAL GETTER COMMANDS\n\n";
		helpBuffer += "Usage:\n";
		helpBuffer += "get OPTION\n\n";
		helpBuffer += "OPTIONS\n";
		helpBuffer += "block/b              -> local node lock status\n";
		helpBuffer += "id/i                 -> local node ID\n";
		helpBuffer += "ip/p                 -> local nodeip address\n";
		helpBuffer += "bypid/bp [VALID_PID] -> current execution node by PID\n";
		helpBuffer += "nodelist/nl          -> list of available nodes\n";
		helpBuffer += "deadnodes/dn         -> list of dead nodes\n";
	}
	else if(helpPage == "3")
	{
		helpBuffer =  "MULTIPLE-NODES GETTER COMMANDS\n\n";
		helpBuffer += "Usage:\n";
		helpBuffer += "get OPTION [COMMA_SEPARATED_NODELIST]\n\n";
		helpBuffer += "OPTIONS\n";
		helpBuffer += "cpus/c    -> cpus\n";
		helpBuffer += "load/l    -> load\n";
		helpBuffer += "mem/m     -> available memory (oM)\n";
		helpBuffer += "rmem/r    -> available memory (Linux)\n";
		helpBuffer += "speed/s   -> speed\n";
		helpBuffer += "status/a  -> status\n";
		helpBuffer += "tmem/t    -> available mem\n";
		helpBuffer += "util/u    -> node utilization\n";
	}
	else if(helpPage == "4")
	{
		helpBuffer =  "LOCAL SETTER COMMANDS\n\n";
		helpBuffer += "Usage:\n";
		helpBuffer += "set OPTION\n\n";
		helpBuffer += "OPTIONS\n";
		helpBuffer += "sblock sb -> set block\n";
		helpBuffer += "ublock ub -> set noblock\n";
		helpBuffer += "decay  d  ->  set job type (decay)\n";
	}
	#ifdef __LIBGTOP
	else if(helpPage == "5")
	{
		helpBuffer =  "LOCAL SYS GETTER COMMANDS\n\n";
		helpBuffer += "Usage:\n";
		helpBuffer += "get OPTION\n\n";
		helpBuffer += "OPTIONS\n";
		helpBuffer += "cpuload/cl          -> load of cpus\n";
		helpBuffer += "cpuinfo/ci          -> contents of cpuinfo file\n";
		helpBuffer += "netload/nl [IFACE]  -> load of the network\n";
		helpBuffer += "pidlist/dl          -> list of pids\n";
		helpBuffer += "proclist/pl         -> TODO\n";
		helpBuffer += "mountlist/ml        -> list of mounted fs\n";
		helpBuffer += "mountusage/ul       -> list of fs usages\n";
		helpBuffer += "uptime/ut           -> uptime and idletime\n";
		helpBuffer += "swapusage/su        -> swap usage\n";
		helpBuffer += "loadaverage/la      -> load average\n";
		helpBuffer += "procstate/ps [PID]  -> proc state\n";
		helpBuffer += "procuid/pu [PID]    -> proc uid and tty infos\n";
		helpBuffer += "procmemory/pm [PID] -> proc memory status\n";
		helpBuffer += "proctime/pt [PID]   -> proc time status\n";
		helpBuffer += "procsignal/pg [PID] -> proc signal status\n";
	}
	#endif
	else
	{
		helpBuffer =  "<ERR>: requested help page doesn't exist.\n";
	}

	return helpBuffer;
}
