/***************************************************************************
*                          clusterSnapshot.cpp  -  description
*                             -------------------
*    begin                : Sam Feb  1 17:08:52 CET 2003
*    copyright            : (C) 2003 by the gomd group at savannah.org/
*    email                : rpons@rinu.org, brian@axehind.com, jp80@libero.it, mosixview@t-online.de
****************************************************************************/

/**
 * NOTES:
 * Created by rejected
 *
 * Contains code to get system infos
 */

#include "clusterSnapshot.h"

/**
 * used to generate HTML table row with alternate colors
 */
void clusterSnapshot::swapHTMLRowColor()
{
	m_isRowEven=!m_isRowEven;
	if(m_isRowEven==true)
	{
		m_rowColor="BGCOLOR='#FFFFF0'";
	}
	else
	{
		m_rowColor="BGCOLOR='#E1FFE1'";
	}
}

/**
 * creates a complete snapshot for the whole cluster (as HTML file format)
 * NOTE: the path of the HTML output file is set in constants.h file
 * @param localNodeIP is the IP address for the local node
 * @param localHostName is the hostname for the local node
 */
void clusterSnapshot::createHTMLSnapshot(string localNodeIP, string localHostName)
{
	time_t rawtime;
	time(&rawtime); //time structs used to get the current time
	m_isRowEven = false;

	//get the openMosix node ID for the local node
	string localNodeId = m_ominfo.getLocalID();


	//START OF HTML PAGE
	m_infoList.push_back("\n<CENTER><H1>CLUSTER SNAPSHOT FOR NODE #" + localNodeId + "</H1><BR>");
	m_infoList.push_back("\n<H2>Generated by GOMD at ");
	m_infoList.push_back(ctime(&rawtime));
	m_infoList.push_back("</H2></CENTER><BR><BR>");
	m_infoList.push_back("\n<FONT SIZE='2'>");
	m_infoList.push_back("\n<TABLE WIDTH='70%' ALIGN='CENTER' BORDER='1'>");


	//OM INFO HTML CODE
	makeHTMLRows("(HOSTNAME:"+localHostName+")");
	makeHTMLRows(m_ominfo.getInfo("cpus",localNodeId,false));
	makeHTMLRows("(GOMD_STATUS:running)");
	makeHTMLRows("(LOCAL_ID:"+localNodeId+")");
	makeHTMLRows(m_ominfo.getInfo("load",localNodeId,false));
	makeHTMLRows(m_ominfo.getLocalBlock());
	makeHTMLRows("(LOCAL_IP:"+localNodeIP+") ");
	makeHTMLRows(m_ominfo.getInfo("mem",localNodeId,false));
	makeHTMLRows(m_ominfo.getInfo("rmem",localNodeId,false));
	makeHTMLRows(m_ominfo.getInfo("speed",localNodeId,false));
	makeHTMLRows(m_ominfo.getInfo("status",localNodeId,false));
	makeHTMLRows(m_ominfo.getInfo("tmem",localNodeId,false));
	makeHTMLRows(m_ominfo.getInfo("util",localNodeId,false));


	//SYSTEM INFO HTML CODE
        #ifdef __LIBGTOP
        makeHTMLRows("(EXTRA_SYS_INFO_SUPPORT:enabled)");

	makeHTMLSection(true,"/proc/cpuinfo");
	makeHTMLRows(cpuinfoFileTweak(m_sysinfo.getCpuinfoFile("   - ",false)));
	makeHTMLSection(false);

	makeHTMLSection(true,"/proc/meminfo");
	makeHTMLRows(m_sysinfo.getMeminfoFile("   - ",false));
	makeHTMLSection(false);

	makeHTMLRows(m_sysinfo.getCpusLoad());

	makeHTMLSection(true,"Load average");
	makeHTMLRows(m_sysinfo.getLoadAverage());
	makeHTMLSection(false);

	makeHTMLSection(true,"Mount List");
	makeHTMLRows(m_sysinfo.getMountList("   - ",false));
	makeHTMLSection(false);

	makeHTMLSection(true,"Mount Usage List");
	makeHTMLRows(m_sysinfo.getMountUsageList("   - ",false));
	makeHTMLSection(false);

	makeHTMLRows(m_sysinfo.getPidList());

	makeHTMLSection(true,"Swap Usage");
	makeHTMLRows(m_sysinfo.getSwapUsage());
	makeHTMLSection(false);

	makeHTMLSection(true,"Uptime Infos List");
	makeHTMLRows(m_sysinfo.getUptime());
	makeHTMLSection(false);
        #endif

	//END OF HTML PAGE
	m_infoList.push_back("\n</TABLE>");
	m_infoList.push_back("\n</FONT>");
	m_infoList.push_back("\n<BR><BR>\n");
}

/**
 * used to create an HTML table row
 * @param isStartSection defines if this is teh first HTML row or not.
 * @param cellTitle is the title of a cell
 */
void clusterSnapshot::makeHTMLSection(bool isStartSection, string cellTitle)
{
	if(isStartSection==true)
	{
		swapHTMLRowColor();
		string ret = "<TR>\n<TD "+m_rowColor+" WIDTH='50%'>"+cellTitle+"</TD>\n<TD "+m_rowColor+" WIDTH='50%'><TABLE>";
		swapHTMLRowColor();
		m_infoList.push_back(ret);
		return;
	}
	else
	{
		m_infoList.push_back("</TABLE></TD>\n</TR>\n");
		return;
	}
}

/**
 * used to handle a nasty situation while reading the /proc/cpuinfo file
 * @param toBeCleaned is a string to be checked
 * @return a "purified" string if the nasty situation is found
 */
string clusterSnapshot::cpuinfoFileTweak(string toBeCleaned)
{
	string::size_type pos;
	string ret = toBeCleaned;

	//cpuinfo file tweak (the "(tm)" word confuses the HTML snapshot creation
	//so we change it with "[tm]")
	pos = ret.find("(tm)");
	if(pos!=string::npos)
	{
		ret.replace(pos,4,"[tm]");
	}
	return ret;
}

/**
 * used to create all the rows for the HTML snapshot.
 * @param toBeCleaned is a string to be parsed and formatted in HTML rows
 */
void clusterSnapshot::makeHTMLRows(string toBeCleaned)
{
	string tmp="";
	bool firstChar= true;

	//swap the color for the next HTML table row
	swapHTMLRowColor();

	//for all the characters in the input string
	for(unsigned int i=0;i<toBeCleaned.size();i++)
	{
		if(toBeCleaned.at(i)=='\n') //new line found!
		{
			m_infoList.push_back(tmp);
			tmp = ""; //clean up buffer
		}
		else if(toBeCleaned.at(i)=='-' || toBeCleaned.at(i)=='\t')
		{
			tmp += "";
		}
		else if(toBeCleaned.at(i)==':')
		{
			tmp += "</TD>\n<TD "+m_rowColor+" WIDTH='50%'>";
		}
		else if(toBeCleaned.at(i)==',')
		{
			tmp += ", ";
		}
		else if(toBeCleaned.at(i)=='_')
		{
			tmp += " ";
		}
		else if(toBeCleaned.at(i)=='(')
		{
			tmp += "<TR>\n<TD "+m_rowColor+" WIDTH='50%'>";
			firstChar=true;
		}
		else if(toBeCleaned.at(i)==')')
		{
			tmp += "</TD>\n</TR>\n";
		}
		else if(toBeCleaned.at(i)!='(' && toBeCleaned.at(i)!=')')
		{
			if (firstChar==false)
			{
				tmp += tolower(toBeCleaned.at(i));
			}
			else
			{
				tmp += toupper(toBeCleaned.at(i));
				firstChar=false;
			}
		}
	}
	m_infoList.push_back(tmp);
}
