/***************************************************************************
*                          connHandler.h  -  description
*                             -------------------
*    begin                : Sam Feb  1 17:08:52 CET 2003
*    copyright            : (C) 2003 by the connHandler group at savannah.org/
*    email                : rpons@rinu.org, brian@axehind.com, jp80@libero.it, mosixview@t-online.de
****************************************************************************/

/**
 * NOTES:
 * Created by openMosixview
 * Last update by rejected
 *
 * Contains the code to handle a single client connection
 */
#ifndef __CONN_HANDLER_H
	#define __CONN_HANDLER_H

	#include <cc++/socket.h>
	#include <cc++/thread.h>
	#include <iostream>
	#include <string>
	#include <list>

	#include "clusterSnapshot.h"
	#include "constants.h"
	#include "gomd2gomd.h"

	#include "ncSocket.h"
	#include "requestHistory.h"
	#include "sysLog.h"
	#include "utils.h"

	//NOTE: if the daemon is compiled with the support for "Extra informations", we
	//include the sysInfo object. This object gets system informations
	//via libgtop2 library (that is required to enable the "Extra inbformations" support)


	#include "globalInitializer.h"

	using namespace std;
	using namespace ost;

	/*********************************/
	/* Socket handler                */
	/*********************************/
	class connHandler : public TCPStream, public Thread
	{
		//vars
		private:
		//internals vars
		char   m_buffer[CONN_HANDLER_MAX_REQUEST_LENGTH];
		string m_localNodeID; //id of this node
		string m_localNodeIP; //ip of this node
		string m_localNodeHostname; //hostname of this node
		string m_remoteClientIP; //ip of the remote client
		string m_lastRequest; //last request sent by the client
		string m_rCmd; //used to store commands
		string m_rRequest; //used to store requests
		string m_rParam; //used to store taget-nodes
		string m_answerBuffer; //used to store the answer to a client request
		long
		int    m_socketTimeout; //socket timeout
		int    m_rFunctionByNumber; //used to store the function to call (call-by-number)
		bool   m_requireCleanup;   //used by chSimpleGC to force the cleanup of this object
		bool   m_remoteRequest; //use to set if the current request is local or remote

		//special objects
		gomd2gomd m_gomd2gomd; //used to handle comms with other connHandlers
		utils m_util; //provides some helper functions
		requestHistory m_requestHistory; //used to store last requests
		sysLog* m_slog; //used to write datas to system logger

		globalInitializer* m_globalInit;
		clusterSnapshot m_clusterSnapshot;

		//functions
		private:
		void run(); //thread loop code

		// history functions
		void repeatLastRequest();
		void repeatUpRequest();
		void repeatDownRequest();
		void printAllRequests();

		//functions
		public:
		connHandler(ncSocket& server);
		void init(globalInitializer* cscoll, sysLog &slog, int socketTimeout, const char *localIP, const char *localHostname);
		~connHandler(); //destructor for connHandler class

		// parse, analyze and execute requests
		void parseRequest(char* clientRequest, string& rCmd, string& rRequest, string& rNode); //parse the request
		int  analyzeOption(char* clientRequest, string& cmd, string& request, string& param); //syntacic validation of client's request
		void fixShortFormatRequest(string& cmd, string& request, string& param); //check short option formats
		bool splitComplexRequest(int rFunctionByNumber, string& rParam); //split complex request in smaller ones
		bool executeSingleRequest(int callFuntionByNumber, string singleTarget=""); //execute a single (and simple) request

		//misc functions
		string securityChecks(string command);
		string executeCommand(string command);
		string executeCommandByNodeID(string command);
		string executeCWCommand(string command);
		string generateSnapshot();

		void sendBannerToClient(); //sens gomd banenr to the client (at the client-conn startup)
		string getLocalIP(); //return local IP
		bool   isCleanupRequired();

	};
#endif // end of __CONN_HANDLER_H


