/***************************************************************************
                          constants.h  -  description
                             -------------------
    begin                : Sam Feb  1 17:08:52 CET 2003
    copyright            : (C) 2003 by the gomd group at savannah.org/
    email                : rpons@rinu.org, brian@axehind.com, jp80@libero.it, mosixview@t-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/* $Id: gomd2gomd.cpp,v 1.10 2004/02/07 22:32:24 rejected Exp $ */

/**
 * NOTES:
 * Created by rejected
 *
 * This class contains the code for gomd2gomd communication.
 */

#include "gomd2gomd.h"

/**
 * opens the connection to remote gomd.
 * @param remoteGomdIP is a string with the IP adress of the remote node.
 * @param port is the integer with the port for remote node.
 * @return true if the conn was succesfully open; false otherwise
 */
bool gomd2gomd::openConn(string remoteGomdIP, int port)
{
	//cout << "<GOMD2GOMD>" << remoteGomdIP << ":" << port << endl;
	return m_libgomd.openConn(remoteGomdIP, port);
} 

/**
 * closes an open connection to remote gomd.
 * @return a message if an error occurred.
 */
string gomd2gomd::closeConn()
{
	return m_libgomd.closeConn();
}

/**
 * requests an info to a remote gomd daemon
 * @param infoToRequest is the info to request to a remote gomd daemon
 * @return the answer of the daemon (just a single line) or a message if an error occurred
 */
string gomd2gomd::getInfo(string infoToRequest)
{
	return m_libgomd.getInfo(infoToRequest);
}

/**
 * requests an info to a remote gomd daemon
 * @param infoToRequest is the info to request to a remote gomd daemon
 * @param endMsg is the string used to signal the end-of-transmission
 * @return the answer of the daemon (multiline) or a message if an error occurred
 */
string gomd2gomd::getMultilineInfo(string infoToRequest, string endMsg)
{
	return m_libgomd.getMultilineInfo(infoToRequest, endMsg);
}

/**
 * this is an atomic-like function used to:
 * 1)open a conn to a remote gomd daemon
 * 2)ask the daemon about an info
 * 3)get the info from the dameon (just one line!)
 * 4)close the connection
 * @param remoteGomdIP is a string with the IP adress of the remote node.
 * @param port is the integer with the port for remote node.
 * @param infoToRequest is the info to request to a remote gomd daemon
 * @return the answer of the daemon (one line) or a message if an error occurred
 */
string gomd2gomd::getInfoAndClose(string remoteGomdIP, int port, string infoToRequest)
{
	string ret;

	if(openConn(remoteGomdIP,port)==false)
	{
		return "[G2G]: cannot open conn to #"+remoteGomdIP+"#";
	}
	ret = getInfo(infoToRequest);
	cout << "[G2G]: answer size (chars): " << ret.size() << endl;
	if(ret.size()==0)
	{
		ret = "[G2G]: no answer from remote gomd!!!";
	}
	closeConn();
	return ret;
}

/**
 * this is an atomic-like function used to:
 * 1)open a conn to a remote gomd daemon
 * 2)ask the daemon about an info
 * 3)get the info from the dameon (multiline)
 * 4)close the connection
 * @param remoteGomdIP is a string with the IP adress of the remote node.
 * @param port is the integer with the port for remote node.
 * @param infoToRequest is the info to request to a remote gomd daemon
 * @param endMsg is the string used to signal the end-of-transmission
 * @return the answer of the daemon (multiline) or a message if an error occurred
 */
string gomd2gomd::getMultilineInfoAndClose(string remoteGomdIP, int port, string infoToRequest, string endMsg)
{
	string ret;

	if(openConn(remoteGomdIP,port)==false)
	{
		return "[G2G]: cannot open conn to #"+remoteGomdIP+"#";
	}
	ret = getMultilineInfo(infoToRequest,endMsg);
	cout << "[G2G]: answer size (chars): " << ret.size() << endl;
	if(ret.size()==0)
	{
		ret = "[G2G]: no answer from remote gomd!!!";
	}
	closeConn();
	return ret;
}
