
/***************************************************************************
                          gomdSearcher.cpp  -  description
                             -------------------
    begin                : Sam Feb  1 17:08:52 CET 2003
    copyright            : (C) 2003 by the gomd group at savannah.org/
    email                : rpons@rinu.org, brian@axehind.com, jp80@libero.it, mosixview@t-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/* $Id: gomdSearcher.cpp,v 1.31 2004/04/11 20:59:01 rejected Exp $ */

/**
 * NOTES:
 * Created by rejected
 *
 * This class contains the code for gomd2gomd communication.
 */

#include "gomdSearcher.h"

/**
 * standard constructor
 * @param ominfo is a pointer to the global omInfo object (created in csCollector class)
 */
gomdSearcher::gomdSearcher(omInfo* ominfo)
{
	if(ominfo == NULL)
	{
		cout << "[RGS]: error: ominfo is NULL!!!"<< endl;
		abort();
	}
	m_omInfo = ominfo;
}

/**
 * set the local IP
 * @param localIP is the string with the local IP required by pingAll() function
 */
void gomdSearcher::setLocalIP(string localIP)
{
	m_localIP = localIP;
}

/**
 * set the highest IP in the local subnet to search for
 * @param highestRangeIP is the highest IP in the local subnet to search for
  */
void gomdSearcher::setMaxRangeIP(unsigned int highestRangeIP)
{
	m_highestRangeIP = highestRangeIP;
}

/**
 * invoke the autodiscovery start (start the thread and detach it)
 */
void gomdSearcher::startSearch()
{
	cout << "[DBG]: startSearch() invoked!" << endl;
	this->detach();
}

/**
 * thread code (here the search starts)
 */
void gomdSearcher::run()
{
	try
	{
		cout << "[RGS]: remote deamons search mode:" << endl;

		if(m_searchByOmDiscd!=true)
		{
			cout << "       => on local network" << endl;
			cout << BLUE << "[RGS]: starting search..."<< NORMAL << endl;
			searchLocalNetworkForDaemons();
		}
		else
		{
			cout << "       => on nodes found by omdiscd" << endl;
			cout << BLUE << "[RGS]: starting search..."<< NORMAL << endl;
			searchNodesFoundByOmdiscdForDaemons(m_omInfo);
		}

		//reset to default search mode (by network)
		m_searchByOmDiscd=false;

		Thread::exit();
	}
	catch( ... )
	{
		cout << "[RGS]: error: exception occurred!"<< endl;
	}
	cout << BLUE << "[RGS]: internal thread killed!!!"<< NORMAL << endl;
}

/**
 * "pings" all remote gomd daemons
 * retunr true if all wen right; false otherwise
 */
bool gomdSearcher::searchLocalNetworkForDaemons()
{
	//the allowed IP range is 1<->254 (broadcast and network IP addresses are forbidden)
	if(m_highestRangeIP<=0 || m_highestRangeIP>=255)
	{
		cout << "[RGS]: illegal max value for IP range to be searched for remote gomds" << endl;
		cout << "[RGS]: obviously remote gomds search cannot be executed." << endl;
		cout << "[RGS]: illegal value: " << m_highestRangeIP << endl;
		return false;
	}

	string subnet;
	string testIP;

	if(m_searchInProgress==true)
	{
		return false;
	}
	m_searchInProgress=true;

	cout << "[RGS]: searching for other gomds..." << endl;
	subnet = m_util.getSubnetFromIP(m_localIP);
	cout << "[RGS]: local subnet: " << subnet << ".x" << endl;
	cout << "[RGS]: highest IP to look for: " << subnet << "." << m_highestRangeIP << endl;

	if(subnet==UTILS_NET_ERROR)
	{
		cout << "[RGS]: cannot get local subnet. Exiting..." << endl;
		return false;
	}

	for(unsigned int i=1; i<=m_highestRangeIP; i++)
	{
		testIP = subnet;
		testIP += ".";
		testIP += m_util.intToString(i);
		cout << pingRemoteGomd(testIP, 9889) << endl;
	}

	m_searchInProgress=false;

	cout << "[RGS]: remote gomd daemons found: " << m_knownGomds.size() <<  endl;
	saveKnownGomdsMapToFile();
	return true;
}

/**
 * searches all remote gomd daemons on nodes already found by omdiscd
 * (openMosix autodiscovery daemon)
 * return true if all went right; false otherwise
 */
bool gomdSearcher::searchNodesFoundByOmdiscdForDaemons(omInfo* m_ominfo)
{
	string tmp;
	string subnet;
	string testIP;

	if(m_searchInProgress==true)
	{
		return false;
	}
	m_searchInProgress=true;


	cout << "[RGS]: searching for other gomds..." << endl;
	subnet = m_util.getSubnetFromIP(m_localIP);
	cout << "[RGS]: local subnet: " << subnet << ".x" << endl;

	m_ominfo->getNodesList(false); //used to fill the nodes array (var member "m_nodesList")
	//cout << "[RGS]: omdiscd found " << m_ominfo->m_nodesNumber << " nodes." << endl;

	if(subnet==UTILS_NET_ERROR)
	{
		cout << "[RGS]: cannot get local subnet. Exiting..." << endl;
		return false;
	}

	for(unsigned int i=0; i<=MAX_REMOTE_NODES_TO_HANDLE; i++)
	{
		tmp = m_ominfo->m_nodesList[i];
		//cout << "=>tmp=" << tmp << endl;
		if(tmp == "[LAST]")
		{
			break;
		}
		testIP = subnet;
		testIP += ".";
		testIP += tmp; //for each node returned by m_ominfo->getNodesList() function
		cout << pingRemoteGomd(testIP, 9889) << endl;
	}

	m_searchInProgress=false;

	cout << "[RGS]: remote gomd daemons found: " << (m_knownGomds.size()-1) <<  endl;
	saveKnownGomdsMapToFile();
	return true;
}


/**
 * "pings" a remote gomd daemon
 * @param IPAddress is a string with the IP of remote gomd
 * @param currentGomdPort is an integer with the IP of remote gomd
 */
string gomdSearcher::pingRemoteGomd(string IPAddress, int currentGomdPort)
{
	string ret = "  - trying IP: " + IPAddress;
	string answer;

	if(m_g2g.openConn(IPAddress,9889)==false)
	{
		ret += " -> \033[31mgomd NOT found\033[0m";
		m_g2g.closeConn();
		return ret;
	}
	string req;
	req += "get id";
	answer = m_g2g.getInfo(req);
	if(answer.size()==0)
	{
		ret += " -> \033[31mgomd NOT found\033[0m";
		m_g2g.closeConn();
		return ret;
	}

	m_util.trim(answer);
	string::size_type pos1 = answer.find(":");
	string::size_type pos2 = answer.find(")");
	if(pos1!=string::npos && pos2!=string::npos)
	{
		answer=answer.substr(pos1+1,pos2-pos1-1);
	}
	ret += " -> \033[32mgomd found!\033[0m";
	m_knownGomds.insert(make_pair(IPAddress,answer));
	m_g2g.closeConn();
	return ret;
}

/**
 * save the found gomd to a file
 * @return a string with the result of the operation
 */
string gomdSearcher::saveKnownGomdsMapToFile()
{
		string valueToWrite;
		map<string,string>::iterator iter;

		m_util.openFile(KNOWN_GOMDS_FILE, false);

		for(iter=m_knownGomds.begin(); iter!=m_knownGomds.end(); ++iter)
		{
			valueToWrite += iter->first;
			valueToWrite += ":";
			valueToWrite += iter->second;
			valueToWrite += "\n";
			if(m_util.writeToOpenFile(valueToWrite)==false)
			{
				m_util.closeFile();
				return "[RGS]: cannot write to known gomds file.";
			}
		}

		m_util.closeFile();

		return "[RGS]: known gomds cache file succesfully updated.";
}

/**
 * searches the known gomd map for the IP address of the specified node ID
 * @param nodeID is a string with the nodeID of remote gomd
 * @return the IP address of the given node ID if available; otherwise return an error message
 */
string gomdSearcher::getNodeIPbyNodeID(string nodeID)
{
		//cout << "<GSEARCHER>: nodeID:" << nodeID << endl;
		if(m_knownGomds.size()==0)
		{
			return "<ERROR>";
		}

		string nodeIP;
		map<string,string>::iterator iter;

		for(iter=m_knownGomds.begin(); iter!=m_knownGomds.end(); ++iter)
		{
			//cout << "<GSEARCHER>: check if nodeID (" << nodeID << ") is equal to an ID from Map (" << iter->second << ")" << endl;
			if(iter->second==nodeID)
			{
				return iter->first;
			}
		}
		return "<ERROR>";
}

/**
 * try loading known gomds map from cache file
 * @return true if the map was succesfully filled; false otherwise
 */
bool gomdSearcher::loadKnownGomdsMapFromFile()
{
	cout << "[RGS]: loading known gomds from cache file..." << endl;
	cout << "[RGS]: opening file: \"" << KNOWN_GOMDS_FILE << "\"" << endl;

	multimap<string,string> knownGomds = m_util.parseSpecialFile(KNOWN_GOMDS_FILE,":");
	multimap<string,string>::iterator iter;

	if(knownGomds.size()==0)
	{
		cout << "[RGS]: cannot read remote gomds cache file!" << endl;
		cout << "[RGS]: cannot fill gomds map!" << endl;
		cout << "[RGS]: you can start autodiscovery manually" << endl;
		cout << "       if you want to find remote gomds." << endl;
		return false;
	}

	m_knownGomds.clear();

	for(iter=knownGomds.begin(); iter!=knownGomds.end(); ++iter)
	{
		m_knownGomds.insert(make_pair(iter->first, iter->second));
		cout << "       -> " << iter->first << ":"<< iter->second << endl;
	}

	if(m_knownGomds.size()>0)
	{
		cout << "[RGS]: found " << m_knownGomds.size() << " remote gomds in cache (dup discarded)." << endl;
		cout << "[RGS]: gomds map succesfully filled." << endl;
		return true;
	}
	else
	{
		cout << "[RGS]: no gomd found in cache file" << endl;
		cout << "[RGS]: gomds map is empty!" << endl;
		cout << "[RGS]: you can start autodiscovery manually" << endl;
		cout << "       if you want to find remote gomds." << endl;
		return false;
	}
}


/**
 * try loading known gomds map from cache file
 * @return true if the map was succesfully filled; false otherwise
 */
void gomdSearcher::appendNodeInfosToKnownGomdsMap(bool isOMEnabled)
{
	string tmp;
	m_omInfo->getNodesList(false);
	
        if(isOMEnabled==true)
        {
                cout << "[RGS]: adding local nodes to known gomds map..." << endl;
                for(unsigned int i=0; i<=MAX_REMOTE_NODES_TO_HANDLE; i++)
		{
			tmp = m_omInfo->m_nodesList[i];
			//cout << "tmp: " << tmp << endl;
			if(tmp == "[LAST]")
			{
				break;
			}
			m_knownGomds.insert(make_pair(m_omInfo->getNodeIDByNodeIP(m_omInfo->m_nodesList[i]), m_omInfo->m_nodesList[i]));
		}
                
        }
        
        cout << "[RGS]: contents of known gomds map..." << endl;
        multimap<string,string>::iterator iter;
        for(iter=m_knownGomds.begin(); iter!=m_knownGomds.end(); ++iter)
        {
                cout << "       -> " << iter->first << ":"<< iter->second << endl;
        }       
}

/**
 * parse the (legacy) openMosix map file (/etc/[open]mosix.map)
 * to get valid entries for the knwon gomd cache file
 * @return true if the map was succesfully filled; false otherwise
 */
bool gomdSearcher::parseOmMapFileAndFillKnownGomdsMap()
{
	cout << "[RGS]: parsing oM map file: '" << OM_NODESMAP_FILE << "'" << endl;
	list<string>::iterator iter;
	string tmp;

	//fill a list with the lines of the (legacy) oM map file
	list<string> fileContents = m_util.readFileEx(OM_NODESMAP_FILE);

	cout << "[RGS]: converting lines to entries for cache map..." << endl;

	//if the file is empty....
	if(fileContents.size()==0)
	{
		cout << "[RGS]: oM map file is empty!" << endl;
		return false; //return...
	}

	//if we're here, the file is not empty
	//for each line of the (legacy) oM map file
	for(iter=fileContents.begin(); iter!=fileContents.end(); ++iter)
	{
		tmp = *iter;

		//try to convert a line to a valid entry for knownGomdsMap
		createKnownGomdMapEntry(tmp);
	}
	return true;
}

/**
 * parse and eventually convert a string to valid knownGomdsMap entry
 * (if required this functions performs a DNS lookup to get valid
 *  IPs instead of hostnames)
 * @param lineToBeParsed is the line to be converted to valid knownGomdsMap entry
 */
void gomdSearcher::createKnownGomdMapEntry(string lineToBeParsed)
{
	int pos=0;
	string tmp;
	string resolve;

	/*
	  this array will contain the 3 required values used
	  to create a valid entry for knownGomdMap

	  CELLS DESCRIPTION
	  cell #0: starting node id
	  cell #1: starting ip (hostname will be automagically resolved as valid IPs)
	  cell #2: nodes (ips) in the range
	*/
	string wantedValues[3];

	//if the line is empty or contains the "comment" symbol...
	if(lineToBeParsed=="" || lineToBeParsed.at(0)=='#')
	{
		return; //...exit!
	}

	StringTokenizer::iterator iter;

	//create tokenizer to parse the inout string
	StringTokenizer st(lineToBeParsed.c_str()," ", false, true);

	try
	{
		//for each token foudn in the input string...
		for(iter=st.begin();iter!=st.end();++iter) //for all the found words...
		{
			tmp = *iter;
			//if the token is not empty and doesn't contain the "tab" character...
			if(tmp!="" && tmp.at(0)!='\t')
			{
				wantedValues[pos]=tmp; //fill the correct cell of the array...
				++pos;
			}
		}
	}
	catch(...) //if any execption/error occurred
	{
		return; //...return!
	}

	//fill cell #0 (starting node id)
	int nodeId = m_util.stringToInt(wantedValues[0]);

	//fill cell #1 (starting ip)
	string startIP = wantedValues[1];

	//if the start ip is an hostname, not a ip (as required)...
	if(m_util.containsLetters(startIP)==true)
	{
		resolve = getIPByHostname(startIP); //...do a DNS lookup
		if(resolve==RGS_CANNOT_RESOLVE_IP) //if the DNS lookup failed...
		{
			cout << "[RGS]: error: DNS lookup failed! (input: "<< startIP << ")" <<endl;
			return; //exit!
		}

		//if we're here, all went right!
		cout << "       (lookup required for host '" << startIP << "' -> " << resolve << ")" << endl;
		startIP = resolve;
	}

	//fill cell #2 (nodes/ips in the range)
	int numOfIps = m_util.stringToInt(wantedValues[2]);

	//get the host number of the start ip
	int hostnumber = m_util.stringToInt(m_util.getHostNumberFromIP(startIP));

	//get the subnet of the start ip
	string subnet = m_util.getSubnetFromIP(startIP);

	//convert nodes ranges to single nodes
	//each node is defined by a pair: IP:OM_NODE_ID
	for(int i=0; i<numOfIps; i++)
	{
		cout << "       -> " << subnet << "." << hostnumber << ":" << nodeId << endl;

		//OK: we can insert the new node to known gomds map!
		m_knownGomds.insert(make_pair(subnet + "." + m_util.intToString(hostnumber), m_util.intToString(nodeId)));
		++hostnumber;
		++nodeId;
	}
}

/**
 * do a DNS reverse lookup (get the hostname of a node from the given IP)
 * @param ipToLookFor is the IP of the machine we're doing the lookup on
 * @return the hostname of the machine with the given IP; an error message if the lookup failed
 */
string gomdSearcher::getHostnameByIP(string ipToLookFor)
{
	//contains host infos
	struct hostent *myhost;

	//will be filled with the input IP
	struct sockaddr_in remoteAddr;

	//if we can't convert the ip to the network format...
	if(inet_aton(ipToLookFor.c_str(), &(remoteAddr.sin_addr))==0)
	{
		cout << "[RGS]: error: inet_aton() failed! (input="<< ipToLookFor.c_str() << ")" <<endl;
		return RGS_CANNOT_RESOLVE_IP; //...return an error code!
	}

	//do the DNS reverse lookup!
	myhost = gethostbyaddr((char*) &remoteAddr.sin_addr,sizeof(remoteAddr.sin_addr),AF_INET);

	//if the DNS lookup failed (gethostbyaddr doenn't set the error variable h_errno)
	if(h_errno)
	{
		cout << "[RGS]: error: gethostbyaddr() failed!"<< endl;
		return RGS_CANNOT_RESOLVE_IP; //...return an error code!
	}

	//if we're here, all went right!
	//return the required hostname!
	return string(myhost->h_name);
}

/**
 * do a DNS lookup (get the IP of a node from the given hostName)
 * @param hostName is the
 * @return the IP associated to the given hostName; an error message otherwise
 */
string gomdSearcher::getIPByHostname(string hostName)
{
	//contains host infos
	struct hostent *myhost;

	//do the DNS lookup
	myhost = gethostbyname(hostName.c_str());

	//if an error occurred...
	if(myhost == NULL)
	{
		return RGS_CANNOT_RESOLVE_IP; //...return an error code!
	}

	//if we're here, all went right!
	//return the IP associated to the given hostName.
	return (inet_ntoa(*((struct in_addr *)myhost->h_addr)));
}
