/***************************************************************************
*                          main.cpp  -  description
*                             -------------------
*    begin                : Sam Feb  1 17:08:52 CET 2003
*    copyright            : (C) 2003 by the gomd group at savannah.org/
*    email                : rpons@rinu.org, brian@axehind.com, jp80@libero.it, mosixview@t-online.de
****************************************************************************/


/**
 * NOTES:
 * Created by openMosixview
 * Last update by rejected
 *
 * here starts the gomd! :)
 */

#include "aclRulebook.h"
#include "clusterSnapshot.h"
#include "connHandler.h"
#include "constants.h"
#include "globalInitializer.h"
#include "ncSocket.h"
#include "sysLog.h"
#include "utils.h"

#include <signal.h>
#include <net/if.h> //iface handling (eth0,...)
#include <sys/ioctl.h>
#include <linux/sockios.h>

//VARS DECLARATION
const char *localIP;            //local address
const char* localHostname;      //local host name
InetAddress localIA;            //local address in InetAddress format
int errCode;                    //used to store error values for getLocalIPAddress() function
long totalNumOfHandledConns = 0; //used to store the number of handled conns
int port	 = 9889;        //used to store the port
long connCounter = 0;	        //connections counter
ncSocket* serverSocket;         //server non-blocking socket used to receive the connections.
string errorMsg	 = "";          //used to return error message
string iface 	 = "eth0";      //used to store iface
string logfile   = "/var/log/gomd.log"; //used to store the port

//daemon flags
bool liveCDOptions = false;
bool noDaemon = false;
bool noLog = false;
bool readOmMap = false;
bool strictSecurity = false;
bool testChpox = false;
int  socketTimeout = 60000;


//global and unique object providing lots of services
//for the whole daemon
//(implements the SINGLETON Design Pattern)
globalInitializer* m_globalInitializer;

sysLog slog(DO_ALL);
utils util;
//END OF VARS DECLARATION


//FUNCTION PROTOTYPES
const char*  getLocalIPAddress(string iface, int* errorCode);
inline bool testOption(const char *stringToTest, const char *longOption, const char *shortOption);
int          daemonize(void);
void         acceptClientConnections(void);
void         activateSignalHandlers(void);
void         bindDaemonToPort(void);
void         enableSpecificOptionsForLiveCDDistrosIfRequired(void);
void         evalCommandLineOptions(int argc, char** argv);
void         logBeforeShuttingDown(void);
void         printUsage(void);
void         readOmMapIfRequired(void);
void         setStrictSecurityModeIfRequired(void);
void         sigIntHandler(int sig);
void         sigQuitHandler(int sig);
//END OF FUNCTION PROTOTYPES


/**
 * here the program starts
 * @param argc is the number of passed params
 * @param argv is an array of strings passed by the user via the command line
 * @return a integer-based exit code
 */
int main(int argc , char **argv)
{

	//1)handle the command line options for the daemon
	evalCommandLineOptions(argc, argv);

	//2)try to bind the port for the daemon
	bindDaemonToPort();

	//3)create the global and unique instance for the globalInitializer object
	m_globalInitializer = globalInitializer::Instance(localIP, localHostname); //get a reference to the SINGLETON object

		if(testChpox) // try registering the INIT process via the chpox mechanism
		{
			m_globalInitializer->testRegisterProcess();
		}

	//4)security mode setting
	setStrictSecurityModeIfRequired();

	//5)enable setting for livecd distros (if required)
	enableSpecificOptionsForLiveCDDistrosIfRequired();

	//6)read openmosix map file (if required)
	readOmMapIfRequired();

	//7)finally start accepting clients connections...
	acceptClientConnections();

}

/**
 * this function creates a daemon. The parent doesn't wait for the child
 * @return an integer-based return code
 */
int daemonize(void)
{
	pid_t pid;
	if ((pid = fork()) == -1)
	{
		perror("fork");
		return EXIT_FAILURE;
	}
	else
	{
		if (pid)
		{ // child code
			std::ofstream writePid(GOMD_PIDFILE);
			if (writePid)
			{
				writePid << pid << endl;
				writePid.close();
			}
			std::exit(EXIT_SUCCESS);
		}
		// parent code
		return EXIT_SUCCESS; //main app exits
	}
}

/**
 * enable the load of some specific live-cd distros
 */
void enableSpecificOptionsForLiveCDDistrosIfRequired(void)
{
	if(liveCDOptions==true)
	{
		m_globalInitializer->enableLivecdSpecificSettings();
	}
}

/**
 * force the load of openMosix map
 */
void readOmMapIfRequired(void)
{
	if(readOmMap==true)
	{
		m_globalInitializer->readOpenMosixMapFile();
	}
}


/**
 * write to stdout how to use the program
 */
void printUsage(void)
{
	cout << "gomd " << GOMD_VERSION << endl;
	cout << endl;
	cout << "Usage: gomd [<options>]" << endl;
	cout << "Options supported:" << endl;
	cout << "\t--help or -h\t\tprints this help then exits" << endl;
	cout << "\t--iface or -i <IFACE>\t\tgets IP address from a specific iface" << endl;
	cout << "\t--livecd or -lc\t\tenables specific options for live-cd distros" << endl;
	cout << "\t--logfile or -l <FILE>\t\tenables logging into a file" << endl;
	cout << "\t--nodaemon or -nd\t\tlaunches the program as a standard app" << endl;
	cout << "\t--port or -p <N>\t\tsets the port to which gomd listens" << endl;
	cout << "\t--readmap or -rm\t\tread oM map file to get network topology of nodes" << endl;
	cout << "\t--security or -s\t\tenables strict security mode" << endl;
	cout << "\t--version or -v\t\tprints program version then exits" << endl;
}

/**
 * SIGINT handler
 */
void logBeforeShuttingDown(void)
{
	//slog.log(LOG_INFO,"[INF]: closing all connections");
	slog.log(LOG_NOTICE,"[INF]: clean exit forced");
}

/**
 * used to get the IP address associated to a specific interface
 * @param iface is the interface (i.e. "eth0")
 * @param errorCode is used to return an error code
 * @return the IP associated to the specified interface, "<ERROR>" otherwise
 */
const char *getLocalIPAddress(string iface, int* errorCode)
{
	struct ifreq ifr;
	struct sockaddr_in *ifaddr;
	int fd;

	fd = socket(AF_INET, SOCK_DGRAM, 0);  //create a socket (for datagrams)
	if ( fd == -1 )
	{
		*errorCode = EXIT_FAILURE;
		cout << "[ERR]: error while opening socket" << endl;
		return "<ERROR>";
	}

	memset(&ifr, 0, sizeof(struct ifreq)); //init the struct to 0
	strncpy(ifr.ifr_name, iface.c_str(), IF_NAMESIZE);

	if ( ioctl(fd, SIOCGIFADDR, &ifr) == -1 )
	{
		*errorCode = EXIT_FAILURE;
		close(fd);
		cout << "[ERR]: error on ioctl" << endl;
		return "<ERROR>";

 	}

	ifaddr = (struct sockaddr_in *)&ifr.ifr_addr;
	close(fd);

	*errorCode = EXIT_SUCCESS;
	return inet_ntoa(ifaddr->sin_addr);
}

/**
 * verify if a command line option is valid or not
 * @param stringToTest is the string to test
 * @param longOption is a valid option in long format
 * @param shortOption is a valid option in short format
 * @return true if the passed string is a valid option, false otherwise
 */
inline bool testOption(const char *stringToTest, const char *longOption, const char *shortOption)
{
	if(strcmp(stringToTest, longOption)==0 ||
	    strcmp(stringToTest, shortOption)==0)
	{
		return true;
	}
	return false;
}

/**
 * used to accept clients connections
 */
void acceptClientConnections(void)
{
	try
	{
		//loop until the socket timeout espires...
		while(serverSocket->isPendingConnection())
		{
			//slog.log(LOG_INFO,"[INF]: handling a new connection for a client");

			//update the stats
			++totalNumOfHandledConns;

			//create a new gomd object to handle a new client connection
			connHandler* myConnHandler = new connHandler(*serverSocket);
			myConnHandler->init(m_globalInitializer, slog, CLIENT_TIMEOUT, localIP, localHostname);
			//slog.log(LOG_INFO,"[INF]: new connHandler created!");

			string strlog = "[INF]: total number of handled conns: ";
			strlog.append(util.intToString(totalNumOfHandledConns));
			slog.log(LOG_INFO, strlog);

			//the object contains/is a thread -> detach the thread!
			myConnHandler->detach(); //the thread is detached (unjoinable thread)
		}
	}
	catch(Socket *socket) //socket errors handling
	{
		int err = socket->getErrorNumber();

		//creating an error string (C++ style)
		errorMsg="";
		errorMsg.append("[ERR]: socket error!");
		errorMsg.append(localIA.getHostname());
		errorMsg.append(":");
		errorMsg.append(util.intToString(port));
		errorMsg.append(" = " + util.intToString(err));

		slog.log(LOG_ERR,errorMsg);
		slog.log(LOG_CRIT,"[ERR]: client socket error: socket failure!");
	}
	catch(...)
	{

	}
}

/**
 * used to parse the comamnd line and to fill daemon flags
 * @param argc is the number of passed params
 * @param argv is an array of strings
 */
void evalCommandLineOptions(int argc, char** argv)
{
	cout << "                   |" << endl;
	cout << ",---.,---.,-.-.,---|" << endl;
	cout << "|   ||   || | ||   |" << endl;
	cout << "`---|`---'` ' '`---'" << endl;
	cout << "`---'" << endl;
	//write an entry to system log
	slog.log(LOG_NOTICE,"[INF]: generic openMosix daemon was started");

	//parses cmd line options
	for (int i=1; i < argc; i++)
	{
		if (testOption(argv[i], "--help","-h"))
		{
			printUsage();
			exit(EXIT_SUCCESS);
		}
		else if (testOption(argv[i], "--version","-v"))
		{
			cout << "Version: " << GOMD_VERSION << endl;
			exit(EXIT_SUCCESS);
		}
		else if (testOption(argv[i], "--port","-p"))
		{
			if (++i < argc)
			{
				if((port=util.stringToInt(argv[i]))<0)
				{
					cout << "[DBG]: invalid character(s) found. Using default port (9889)." << endl;
					port=9889;
				}
			}
		}
		else if (testOption(argv[i], "--iface", "-i"))
		{
			if (++i < argc)
			{
				iface.erase();
				iface.append(argv[i]);
			}
		}
		else if (testOption(argv[i], "--logfile", "-l"))
		{
			if (++i < argc)
			{
				logfile= argv[i];
			}
		}
		else if (testOption(argv[i], "--livecd", "-lc"))
		{
			liveCDOptions = true;
		}
		else if (testOption(argv[i], "--nodaemon", "-nd"))
		{
			noDaemon = true;
		}
		else if (testOption(argv[i], "--nolog", "-nl"))
		{
			noLog = true;
		}
		else if (testOption(argv[i], "--readmap", "-rm"))
		{
			readOmMap = true;
		}
		else if (testOption(argv[i], "--sockettimeout", "-st"))
		{
			if(++i < argc)
			{
				if((socketTimeout = util.stringToInt(argv[i]))<0)
				{
					cout << "[DBG]: invalid character(s) found. Using default socket timeout " << socketTimeout << "." << endl;
					socketTimeout = 60000;
				}
			}
		}
		else if (testOption(argv[i], "--security", "-s"))
		{
			strictSecurity = true;
		}
		else if (testOption(argv[i], "--test-chpox", "-tc"))
		{
			testChpox = true;
		}
	}

 	if (!port) //if no port was specified
	{
		printUsage();
	}

	if(!noDaemon) // if the user forces no daemon mode...
	{
		daemonize();
 	}

	if(noLog) // if logging was desabled by command line option...
	{
		slog.setOperationsToDo(PRINT_ONLY); //print messages only on console
	}

}

/**
 * tries to bin a port for the daemon
 */
void bindDaemonToPort(void)
{
	//get the local ip from a specified interface (default is "eth0")
	localIP = getLocalIPAddress(iface, &errCode);
	if(errCode==EXIT_FAILURE) //if an error occurred while getting the ip...
	{
		slog.log(LOG_CRIT,"[ERR]: iface eth0 doesn't exist -> cannot get IP address. Exiting...");
		exit(EXIT_FAILURE);
	}
	else
	{
		localIA = InetAddress(localIP); //convert local string-formatted IP to InetAddress format
		localHostname = localIA.getHostname(); //get local hostname
	}

	slog.log(LOG_NOTICE,"[INF]: binding port " + util.intToString(port) + " on host " + localIA.getHostname());

	try
	{
		//create a socket to handle multiple connections
		serverSocket = new ncSocket(localIA, port,connCounter); //create a socket
	}
	catch(Socket *socket) //socket errors handling
	{
		int err = socket->getErrorNumber();

		//creating an error string (C++ style)
		errorMsg="";
		errorMsg.append("[ERR]: socket error!");
		errorMsg.append(localIA.getHostname());
		errorMsg.append(":");
		errorMsg.append(util.intToString(port));
		errorMsg.append(" = " + util.intToString(err));

		slog.log(LOG_ERR,errorMsg);

		if(err == Socket::errBindingFailed)
		{
			slog.log(LOG_CRIT,"[ERR]: bind error; another program is currently listening on this port");
			exit(EXIT_FAILURE);
		}
	}
}

/**
 * handles strict-security mode activation
 */
void setStrictSecurityModeIfRequired(void)
{
	//if strict-security mode is enabled...
	if(strictSecurity==true)
	{
		//try to set the strict-security mode...
		if (m_globalInitializer->m_security.setStrictSecurity()==false) //if the strict-security mode cannot be set...
		{
			slog.log(LOG_CRIT,"[ERR]: cannot set required strict security mode");
			::exit(EXIT_FAILURE);
		}
		else //ok. strict security mode was succesfully set!
		{
			slog.log(LOG_INFO,"[INF]: strict security mode succesfully set");
		}
	}
}



