/***************************************************************************
*                          omInfo.cpp  -  description
*                             -------------------
*    begin                : Sam Feb  1 17:08:52 CET 2003
*    copyright            : (C) 2003 by the gomd group at savannah.org/
*    email                : rpons@rinu.org, brian@axehind.com, jp80@libero.it, mosixview@t-online.de
****************************************************************************/

/**
 * NOTES:
 * Created by rejected
 *
 * Contains code to get system infos
 */

#include "omInfo.h"

/**
 * returns a specific information for a specific node
 * @param requiredInfo is the specific requested info-file
 * @param nodeId is the target-node
 * @param printNodeID specifies if to print (or not to print) the node ID
 * @return a string with the requested info
 */
string omInfo::getInfo(string requiredInfo, string nodeId, bool printNodeID)
{
        string tmpstr;
	string ret;
	string valueFromFile;

        tmpstr=tmpstr + HPC_NODES_DIRECTORY + "/" + nodeId + "/" + requiredInfo;
	//REMOVE_FOR_DEBUG *tcp() << "<DBG>: " << tmpstr << endl;

	//convert all chars to "upper" case
	m_util.ucase(requiredInfo);

	ret = "(";
	ret += requiredInfo;
	if(printNodeID==true)
	{
		ret += "_OF_NODE_#";
		ret += nodeId;
	}
	ret += ":";
        valueFromFile = m_util.readLineFromFile(tmpstr);
	//cout << "valueFromFile #" << valueFromFile << "#" << endl;
	if(valueFromFile==UTILS_IO_ERROR)
	{
		valueFromFile = OMINFO_OM_NOT_STARTED_OR_NODE_NOT_CONFIGURED;
	}
	ret += valueFromFile;
	ret += ")";
	return ret;
}

/**
 * returns the list of available nodes
 * @return the list of available nodes
 */
string omInfo::getNodesList(bool showNote=false)
{
	string ret;
        string tmpstr;
	string currFile;
	int n=0;

	try
	{
		DIR *directory; //directory handle
		struct dirent *info; //directory entry
		directory = opendir(HPC_NODES_DIRECTORY); //open HPC iface nodes directory
		if(directory==NULL)
		{
			//the oM dir structure wasn't found => this is not an oM node or openMosix is not configured/started!
			return OMINFO_OM_NOT_STARTED_OR_NODE_NOT_CONFIGURED;
		}

		while(((info=readdir(directory))!=NULL) && n<MAX_REMOTE_NODES_TO_HANDLE) //read the directory (read each file included in that directory)
		{
			currFile = string(info->d_name); //get a file name
			if(currFile.compare(".") && currFile.compare("..")) //no special directories allowed...
			{
				m_nodesList[n] = currFile;
				++n;
				tmpstr = tmpstr + "," + currFile; //add the found file to the node list
			}
		}
		m_nodesList[n] = "[LAST]";
		//cout << "Added LAST=> cell #" << n << endl;
		tmpstr = tmpstr.substr(1); //first comma will be removed
		closedir(directory); //close HPC iface nodes directory

		if(showNote==true)
		{
			ret = "(AVAILABLE_NODE_LIST:"+ tmpstr + ")";
		}
		else
		{
			ret = tmpstr;
		}
	}
	catch(...)
	{
		ret="";
	}
	return ret;
}

/**
 * returns the IP of a specified node (if available)
 * @return the IP of a specified node (if available)
 */
string omInfo::getNodeIDByNodeIP(string nodeId)
{
        string ret;
        list<string> cmdsList = m_util.parseCommand("mosctl whois "+nodeId);
	//cout << "mosctl whois "+nodeId << endl;
        list<string> r = m_util.execCommandAndReturnChildOutput(cmdsList, NULL);
        
        if(r.size()==1)
        { 
                list<string>::iterator iter;
                iter = r.begin();
                ret = *iter;
                if(ret.size()<15)
                {       
                        return ret;
                }
        }
        return "ERROR";
}

/**
 * returns the list of dead nodes
 * @return the list of dead nodes
 */
string omInfo::getDeadNodesList()
{
	string nl = getNodesList(false); //get the nodes list

	if(nl=="")
	{
		return "";
	}

	string deadNodes="";
	StringTokenizer st(nl.c_str(),",");
	StringTokenizer::iterator i;
	for(i=st.begin();i!=st.end();++i) //for all the found nodes...
	{
		//*tcp() << *i << endl;
		string tmpValue = getInfo("load", *i); //...try to get node load
		//cout << tmpValue << endl;
		if(tmpValue.find("-101")!=string::npos) //if we cannot find a valid value (==the node is not active)
		{
			deadNodes.append(*i); //append this node to the list of dead nodes
			deadNodes.append(",");
		}
	}
	deadNodes=deadNodes.substr(0,(int)deadNodes.size()-1); //remove the last ","
	return "(DEAD_NODES_LIST:"+ deadNodes + ")"; //return dead nodes list
}

/**
 * returns current node of execution for a process via its PID
 * @param pid is the pid of a process
 * @return current execution node for the process
 */
string omInfo::getNodeByPID(string pid)
{
	string tmpstr; //used to store proc system directory
	string ret;
        tmpstr = tmpstr + "/proc/"+pid+"/where";
	ret = m_util.readLineFromFile(tmpstr);
	if(ret=="0")
	{
		ret = "[this node: no migration!]";
	}
	return ret;
}

/**
 * returns the ID of the local node
 * @return a string with the ID of the local node
 */
string omInfo::getLocalID()
{
        string tmpstr;
        tmpstr.append(HPC_CLUSTER_DIRECTORY);
	tmpstr.append("/admin/mospe");
	
        string ret = m_util.readLineFromFile(tmpstr);
	if (ret=="0") 
	{
		ret=UTILS_IO_ERROR;
	}	
	return ret;
}



/**
 * returns the block status of the local node
 * @return a string with the block status of the local node
 */
string omInfo::getLocalBlock()
{
	string tmpstr;
	tmpstr=tmpstr + HPC_CLUSTER_DIRECTORY + "/admin/block";
	return "(LOCAL_BLOCK:"+ m_util.readLineFromFile(tmpstr) + ")";
}

/**
 * sets the block-status for local oM node
 * @return a string message with the result of the operation
 */
string omInfo::setLocalBlock()
{
        string tmpstr;
        tmpstr=tmpstr + HPC_CLUSTER_DIRECTORY + "/admin/block";
        if (m_util.writeToFile(tmpstr, 1))
        {
                return "(SET_LOCAL_BLOCK: succefully set openMosix block";
        }
        else
        {
                return "(SET_LOCAL_BLOCK: could not set openMosix block";
        }
}

/**
 * unsets the block-status for local oM node
 * @return a string message with the result of the operation
 */
string omInfo::setLocalNoblock()
{
        string tmpstr;
        tmpstr=tmpstr + HPC_CLUSTER_DIRECTORY + "/admin/block";
        if (m_util.writeToFile(tmpstr, 0))
        {
                return "succefully set openMosix noblock";
        }
        else
        {
                return "<ERROR>: could not set openMosix noblock";
        }
}

/**
 * sets decay type for local oM node
 * @param typeOfJob is a valid job-type file
 * @return a string message with the result of the operation
 */
string omInfo::setLocalDecay(string typeOfJob)
{
        string tmpstr;
	tmpstr=tmpstr + HPC_CLUSTER_DIRECTORY + "/decay/clear"; //clear decay value
	m_util.writeToFile(tmpstr, 1);

	tmpstr.erase(); //clear up
	tmpstr=tmpstr + HPC_CLUSTER_DIRECTORY + "/decay/"+typeOfJob; //set the new decay value
	if (m_util.writeToFile(tmpstr, 1))
	{
		return "succefully set job type: "+typeOfJob;
	}
	else
	{
		return "<ERROR>: could not set job type: '"+typeOfJob+"'";
	}
}
