/***************************************************************************
*                          requestHistory.cpp  -  description
*                             -------------------
*    begin                : Sam Feb  1 17:08:52 CET 2003
*    copyright            : (C) 2003 by the gomd group at savannah.org/
*    email                : rpons@rinu.org, brian@axehind.com, jp80@libero.it, mosixview@t-online.de
****************************************************************************/

/**
 * NOTES:
 * Last update by rejected
 *
 * Contains code to handle an history of the request from a client
 */

 #include "requestHistory.h"

 /**
 * default constructor
 */
requestHistory::requestHistory()
{
	m_lastPos = 0;
}

/**
 * used to append a request to the history buffer
 * @param requestToAppend is the request to append
 */
void requestHistory::append(string requestToAppend)
{
	if(requestToAppend.find("get lastrequest")!=string::npos ||
	   requestToAppend.find("get lr")!=string::npos ||
	   requestToAppend.find("get 30")!=string::npos ||
	   requestToAppend.find("get allrequests")!=string::npos ||
	   requestToAppend.find("get ar")!=string::npos ||
	   requestToAppend.find("get 31")!=string::npos)
	{
		//avoid appending "repeat last request" string
		//last (valid) request remains the last (valid) request! :)
		return;
	}
	//REMOVE_FOR_DEBUG else
	//REMOVE_FOR_DEBUG {
	//REMOVE_FOR_DEBUG *tcp() << "" << endl;
	//REMOVE_FOR_DEBUG *tcp() << "*******************" << endl;
	//REMOVE_FOR_DEBUG *tcp() << "* REQUEST HISTORY " << endl;
	//REMOVE_FOR_DEBUG *tcp() << "*******************" << endl;
	//REMOVE_FOR_DEBUG *tcp() << "Cannot insert this request into the history buffer" << endl;
	//REMOVE_FOR_DEBUG }

	if(m_buffer.size()==COMMAND_HISTORY_SIZE)
	{
		m_buffer.pop_back();
	}
	m_buffer.push_front(requestToAppend);
}

/**
 * used to get a specific request from the history buffer
 * @param index is the cell index of the request in the history buffer
 * @return the string, at the specified cell index, from the history buffer (with the wanted request)
 */
string requestHistory::get(unsigned int index)
{
	if (index>=0 && index<=m_buffer.size() && !m_buffer.empty())
	{
		m_bufferIter= m_buffer.begin();
		advance(m_bufferIter,index);
		return *m_bufferIter;
	}
	return "<ERROR>";
}

/**
 * used to get the last request from the history buffer
 * @return the first string from history buffer (with the wanted request)
 */
string requestHistory::getLast()
{
	//cout << m_buffer[0] << endl;
	if(m_buffer.size()==0)
	{
		return "";
	}
	return m_buffer.front();
}

/**
 * used to scroll up the history
 * @return required history entry
 */
string requestHistory::getUp()
{
	if(m_lastPos >= m_buffer.size())
	{
		m_lastPos = 0;
	}
	string ret = get(m_lastPos);
	++m_lastPos;
	return ret;
}

/**
 * used to scroll down the history
 * @return required history entry
 */
string requestHistory::getDown()
{

	if(m_lastPos < 0)
	{
		m_lastPos = m_buffer.size() - 1;
	}
	string ret = get(m_lastPos);
	--m_lastPos;
	return ret;

}
