/***************************************************************************
*                          scxCmdsbook.cpp  -  description
*                             -------------------
*    begin                : Sam Feb  1 17:08:52 CET 2003
*    copyright            : (C) 2003 by the gomd group at savannah.org/
*    email                : rpons@rinu.org, brian@axehind.com, jp80@libero.it, mosixview@t-online.de
****************************************************************************/

/**
 * NOTES:
 * Last update by rejected
 *
 * Contains code to handle secured Cluster-wide Execution
 */

#include "scxCmdsbook.h"

/**
 * constructor for class scxCmdsbook
 * @param localIP contains the IP of local node
 */
scxCmdsbook::scxCmdsbook(string localIP)
{
	m_localIP = localIP;
}

/**
 * used to fill the SCX map with the contents of the SCX configuration file (commands book)
 * @param scxfile is the file with the SCX configuration file (commands book)
 */
void scxCmdsbook::fillScxMap(string scxFile)
{
	cout  << "[SCX]: opening file: \"" << scxFile << "\""  << endl;
	m_scxMap = new multimap<string,string>(m_util.parseSpecialFile(scxFile,":"));
	if (m_scxMap->size()==0)
	{
		cout  << "[SCX]: scx commands book file is empty!"  << endl;
		cout  << "[SCX]: secured cluster-wide execution feature is disabled." << endl;
	}
	else
	{
		cout << "[SCX]: commands map succesfully filled." << endl;
	}
}

/**
 * used to get the SCX map
 * @return the SCX map
 */
multimap<string,string>	scxCmdsbook::getScxMap()
{
	return *m_scxMap;
}

/**
 * prints the contents of the SCX map
 */
void scxCmdsbook::printScxMap()
{
	if(m_scxMap->size()==0)
	{
		return;
	}

	cout << "[SCX]: contents of commands map..." << endl;
	multimap<string,string>::iterator m_iter;
	for(m_iter = m_scxMap->begin(); m_iter != m_scxMap->end(); ++m_iter)
	{
		cout << "       -> " << m_iter->first << ":"<< m_iter->second << endl;
	}
}

/**
 * used set the default behaviour (accept or block all the commands) if
 * one of the two special macros are defined
 */
void scxCmdsbook::setScxDefaultBehaviour()
{
	if(m_scxMap->find("SCX_ALL_CMDS_ALLOWED")!=m_scxMap->end())
	{
		cout << "[SCX]: from now all cmds are allowed!" << endl;
		m_allow_all_cmds=true;
	}

	if(m_scxMap->find("SCX_ALL_CMDS_BLOCKED")!=m_scxMap->end())
	{
		cout << "[SCX]: from now all cmds are blocked!" << endl;
		m_block_all_cmds=true;
	}
}

/**
 * used to search the SCX map to see is a command is included
 * @param toBeFound is the string with the command to be searched
 * @return true if the command is presnet in the SCX map; false otherwise
 */
bool scxCmdsbook::searchScxMapForCommand(string toBeFound)
{
	string tmp = toBeFound;
	m_util.trim(tmp);

	cout << "[SCX]: search command: #" << tmp << "#" << endl;

	if(m_block_all_cmds==true)
	{
		cout << "[SCX]: all cmds blocked by default." << endl;
		return false;
	}
	else if(m_allow_all_cmds==true)
	{
		cout << "[SCX]: all cmds allowed by default." << endl;
		return true;
	}

	if(m_scxMap->find(tmp)==m_scxMap->end())
	{
		cout << "[SCX]: cannot find command in the map!" << endl;
		return false;
	}

	cout << "[SCX]: command was found in the map (perfect match)!" << endl;
	return true;
}
