/***************************************************************************
*                          security.cpp  -  description
*                             -------------------
*    begin                : Sam Feb  1 17:08:52 CET 2003
*    copyright            : (C) 2003 by the gomd group at savannah.org/
*    email                : rpons@rinu.org, brian@axehind.com, jp80@libero.it, mosixview@t-online.de
****************************************************************************/


/**
 * NOTES:
 * Created by rejected
 * Last update by rejected
 *
 * Contains the code to handle the security of gomd
 */

#include "security.h"

/**
 * used to set the security level
 * @param isStrictSecurityEnabled set if the strict-security mode is enabled or disabled
 */
void security::setSecurityLevel(bool isStrictSecurityEnabled)
{
	m_isStrictSecurityEnabled = isStrictSecurityEnabled;
}

/*
 * verify the if gomd has root privileges or standard-user ones
 * @return true is gomd was launched with root permissions; false otherwise
 */
bool security::isRoot()
{
	//UID and GID are system numbers used to identify an user and
	//its running processes: gomd, as program, gets different privileges
	//if you run it as standard user or root!

	//some programs during execution can change temporarily uid and gid (for example: if
	//they need special privileges for some syscalls, they can "switch" to root privileges)
	//EUID and EGID (effective UID and GID) are the current uid and gid for a program.

	long uid 	     = (long)getuid();  //get uid
	long currentUid      = (long)geteuid(); //get effective (current) uid
	long gid 	     = (long)getgid();  //get gid
	long currentGid      = (long)getegid(); //get effective (current) gid

	if(uid==0 || currentUid==0 || //if the user is running gomd as root..
	   gid==0 || currentGid==0)
	{
		return true; //root!
	}
	return false; //standard user!
}

/*
 * verify the if a comamnd is "blacklisted" and cannot be executed if gomd
 * was launched with strict-security mode
 * @param toBeChecked is the string to be checked
 * @return true if the input string contains  a blacklisted command; false otherwise
 */
bool security::isBlackListedCommand(string toBeChecked)
{
	if(
   	   toBeChecked.find("#!")!=string::npos ||
	   toBeChecked.find("fdisk")!=string::npos ||
   	   toBeChecked.find("gomd")!=string::npos ||
   	   toBeChecked.find("halt")!=string::npos ||
	   toBeChecked.find("kill")!=string::npos ||
	   toBeChecked.find("killall")!=string::npos ||
	   toBeChecked.find("nc")!=string::npos ||
	   toBeChecked.find("nmap")!=string::npos ||
	   toBeChecked.find("ping")!=string::npos ||
   	   toBeChecked.find("reboot")!=string::npos ||
	   toBeChecked.find("sh")!=string::npos ||
   	   toBeChecked.find("shutdown")!=string::npos
	   )
	{
		return true;
	}
	return false;
}

/**
 * checks if openMosix is enabled on the local node
 * @return true if openMosix is enabled on local node; false otherwise
 */
bool security::isOMEnabled()
{
	omInfo test;
	if (test.getLocalID()!="0")
	{
		return true;
	}
	return false;
}

/**
 * used to change the current working directory to a given directory
 * @param newCurrWorkingDir is the directory to be set as current working directory
 * @return true if the change was successfully done; false otherwise
 */
bool security::changeCurrWorkingDir(string newCurrWorkingDir)
{
	char currDir[256];
	struct stat infos;

	errno = 0;

	if( !getcwd(currDir, sizeof(currDir)))
	{
		cout << "[SEC]: cannot get current working directory." << endl;
		return false;
	}
	//cout << "[SEC]: current working directory: " << currDir << endl;


	//cout << "[SEC]: check for " << newCurrWorkingDir << ": ";
	if(mkdir(newCurrWorkingDir.c_str(), S_IRUSR|S_IWUSR|S_IXUSR)!=0)
	{
		//cout << " yet present." << endl;
	}
	else
	{
		//cout << " created." << endl;
	}

	//cout << "[SEC]: change current working directory." << endl;
	if(chdir(newCurrWorkingDir.c_str())!=0)
	{
		cout << "[SEC]: cannot set current working directory to: " << newCurrWorkingDir << endl;
		cout << "[SEC]: err description: " << strerror(errno) <<endl;
		return false;
	}
	//cout << "[SEC]: new current working directory: " << newCurrWorkingDir << endl;

	if(stat(newCurrWorkingDir.c_str(), &infos)!=0)
	{
		cout << "[SEC]: cannot get stat infos for directory: " << newCurrWorkingDir << endl;
		cout << "[SEC]: err description: " << strerror(errno) <<endl;
		return false;
	}
	//cout << "[SEC]: directory: owner=" << infos.st_uid << " group=" << infos.st_gid << endl;

	return true;
}

/**
 * used to set the strict security mode
 * @return true if the set was successfully done; false otherwise
 */
bool security::setStrictSecurity()
{

	m_isStrictSecurityEnabled = true;

	cout << "[SEC]: strict security mode is enabled." << endl;
	cout << "[SEC]: performing security checks." << endl;
	cout << "       -> gomd is run with root privilegs: ";
	if (isRoot()==true)
	{
		cout << "yes." << endl;
		cout << "       -> gomd cannot be run with root permissions" << endl;
		cout << "          if strict security mode is enabled!" << endl;
		cout << "[SEC]: cannot continue security checking." << endl;
		return false;
	}
	else
	{
		cout << "no. => ok!" << endl;
	}

	cout << "       -> setting fake dir (" << SEC_FAKE_DIR << ") as cwd: ";
	if (changeCurrWorkingDir(SEC_FAKE_DIR)==false)
	{
		cout << "error." << endl;
		cout << "[SEC]: cannot continue security checking." << endl;
		return false;
	}
	else
	{
		cout << "ok." << endl;
	}

	cout << "       -> check if openMosix is enabled: ";
	if (isOMEnabled()==false)
	{
		cout << "no." << endl;
		cout << "[SEC]: openMosix is disabled or the node is not configured." << endl;
		return false;
	}
	else
	{
		cout << "yes." << endl;
	}
	return true;
}
