/***************************************************************************
*                          sysInfo.cpp  -  description
*                             -------------------
*    begin                : Sam Feb  1 17:08:52 CET 2003
*    copyright            : (C) 2003 by the gomd group at savannah.org/
*    email                : rpons@rinu.org, brian@axehind.com, jp80@libero.it, mosixview@t-online.de
****************************************************************************/


/**
 * NOTES:
 * Created by rejected
 * Last update by rejected
 *
 * Contains the code to get system infos
 */

#include "sysInfo.h"

/**
 * return the contents of /proc/cpuinfo file
 * @param lineStart is an optionl string placed at the beginning of the line
 * @param printBanner sets if the output need a banner or not
 * @return a string with the contents of /proc/cpuinfo file
 *  (pairs <INFO,VALUE> are sorted alphabetically)
 */
string sysInfo::getCpuinfoFile(string lineStart, bool printBanner)
{
	string cpuinfoContents;
	list<string> cpuinfoFileList; //STL list of the pairs <INFO,VALUE>
	list<string>::iterator iter; //iterator for the multimap

	//invoke parsing of the file. The results will be placed into the STL multimap
	cpuinfoFileList = m_util.readFileEx("/proc/cpuinfo");

	if(printBanner==true)
	{
		cpuinfoContents = "(CPUINFO_FILE:contents)";
	}

	//now we create a single formatted line with all the contents of the multimap
	for(iter = cpuinfoFileList.begin(); iter != cpuinfoFileList.end(); ++iter)
	{
		cpuinfoContents += lineStart;
		cpuinfoContents += "(";
		cpuinfoContents += *iter;
		cpuinfoContents += ")";
	}

	//return the string with all the (formatted) contents of /proc/cpuinfo file
	return cpuinfoContents;
}

/**
 * return the contents of /proc/meminfo file
 * @param lineStart is an optionl string placed at the beginning of the line
 * @param printBanner sets if the output need a banner or not
 * @return a string with the contents of /proc/meminfo file
 *  (pairs <INFO,VALUE> are sorted alphabetically)
 */
string sysInfo::getMeminfoFile(string lineStart, bool printBanner)
{
	//invoke parsing of the file. The results will be placed into the STL multimap
	//return m_util.readFile("/proc/meminfo" , "(MEMINFO_FILE:" ,")");

	unsigned int line=0;
	string meminfoContents;
	list<string> meminfoFileList; //STL list of the pairs <INFO,VALUE>
	list<string>::iterator iter; //iterator for the multimap

	//invoke parsing of the file. The results will be placed into the STL multimap
	meminfoFileList = m_util.readFileEx("/proc/meminfo");

	if(printBanner==true)
	{
		meminfoContents = "(MEMINFO_FILE:contents)";
	}


	//now we create a single formatted line with all the contents of the multimap
	for(iter = meminfoFileList.begin(); iter != meminfoFileList.end(); ++iter)
	{
		if(line>2)
		{
			meminfoContents += lineStart;
			meminfoContents += "(";
			meminfoContents += *iter;
			meminfoContents += ")";
		}
		++line;
	}

	//return the string with all the (formatted) contents of /proc/meminfo file
	return meminfoContents;
}

/**
 * return the uptime and the idletime for the local machine
 * @return a formatted string with the uptime and the idletime of the local machine
 */
string sysInfo::getUptime()
{
	string ret;
	string uptime = m_util.readLineFromFile("/proc/uptime");
	string::size_type pos = uptime.find(" ");

	ret += "(UPTIME:";
	ret += uptime.substr(0,pos);
	ret += ")(IDLETIME:";
	ret += uptime.substr(pos+1);
	ret += ")";
	return ret;
}

/**
 * return the number of cpus for local node
 * @return the number of cpus for the local node
 */
int sysInfo::getCpusNumber()
{
	return (m_util.getSubStringOccurInFile("/proc/stat", "cpu") -1);
}

/**
 * return a list of all the identifiers of the running processes (aka PIDs)
 * @param pidArray is a pointer to an array of integers. It will be filled by this function.
 * @return a formatted string with the all the PIDs
 */
string sysInfo::getPidList(list<int>* pidList)
{
	string ret;
        string tmpstr;
	string procDirName;
	int    procDirNumber;

	DIR *directory; //directory handle
	struct dirent *info; //directory entry
	int n=0;

	try
	{
		directory = opendir("/proc"); //open HPC iface nodes directory
		if(directory==NULL)
		{
			//the oM dir structure wasn't found => this is not an oM node or openMosix is not configured/started!
			return "<ERROR>: can't open /proc directory.";
		}

		if(pidList!=NULL)
		{
			pidList->clear();
		}

		while(((info=readdir(directory))!=NULL)) //read the directory (read each file included in that directory)
		{
			procDirName = string(info->d_name); //get a file name
			procDirNumber = m_util.stringToInt(procDirName);
			if(procDirNumber>0)
			{
				if(pidList!=NULL)
				{
					pidList->push_back(procDirNumber);
				}
				++n;
				tmpstr = tmpstr + "," + procDirName; //add the found file to the node list
			}
		}

		tmpstr = tmpstr.substr(1); //first comma will be removed
		closedir(directory); //close HPC iface nodes directory

		ret = "(PID_LIST:"+ tmpstr + ")";
	}
	catch(...)
	{
		ret="<ERROR>: exception occurred.";
	}
	return ret;
}



//CODE USING LIBGTOP

/**
 * return the load for each installed cpu (for local node)
 * @return a formatted string with the load value for each installed cpu
 */
string sysInfo::getCpusLoad()
{
	string cpuLoadValue =""; //return string
	int numberOfCpus; //used to store the number of installed cpu
	glibtop_cpu cpu; //libgtop cpu object

	numberOfCpus = getCpusNumber(); //get the number of cpus for local node

	//each array contains the value of a specific
	//system info for each installed cpu
	int *user      = new int[numberOfCpus];
	int *nice      = new int[numberOfCpus];
	int *sys       = new int[numberOfCpus];
	int *load      = new int[numberOfCpus];
	int *total     = new int[numberOfCpus];
	int *idle      = new int[numberOfCpus];
	int *frequency = new int[numberOfCpus];

	//these array are static because are used to store previus values
	//(used to get current values)
	static int *oldload    = new int[numberOfCpus];
	static int *oldtotal   = new int[numberOfCpus];

	//invoking libgtop function to get the values for _all_ the installed cpus
	glibtop_get_cpu(&cpu);


	//NOTE: libgtop fills "cpu.[INFO]" attributes if there is only a cpu.
	//      If there are more than a single cpu, libgtop fills "cpu.[xINFO]" attributes.

	if (numberOfCpus==1) //there is only a single cpu
	{
		//fill the single cell of the arrays with the values for the single cpu
		user[0]      =  cpu.user;
		nice[0]      =  cpu.nice;
		sys[0]       =  cpu.sys;
		total[0]     =  cpu.total;
		idle[0]      =  cpu.idle;
		frequency[0] =  cpu.frequency;

		load[0]=user[0]+nice[0]+sys[0];

		//compute the load of cpu (uses old values to do it)
		cpuLoadValue += "(LOAD_OF_CPU_#0:"; //format the results
		if (total[0] != oldtotal[0])
		{
			cpuLoadValue += m_util.intToString(100 * (load[0] - oldload[0]) / (total[0] - oldtotal[0]));
		}
		else
		{
			cpuLoadValue += m_util.intToString(0);
		}
		cpuLoadValue += ")";

		//set old values
		oldload[0] = load[0];
		oldtotal[0] = total[0];
	}
	else //there are more than a single cpu (2+)
	{
		for(int i=0;i<numberOfCpus;i++) //for each installed cpu...
		{
			//fill the i-th cell of the arrays with the values for the i-th cpu
			user[i]      =  cpu.xcpu_user[i];
			nice[i]      =  cpu.xcpu_nice[i];
			sys[i]       =  cpu.xcpu_sys[i];
			total[i]     =  cpu.xcpu_total[i];
			idle[i]      =  cpu.xcpu_idle[i];
			frequency[i] =  cpu.frequency;

			load[i]=user[i]+nice[i]+sys[i];

			//compute the load of cpu (uses old values to do it)
			cpuLoadValue += "(LOAD_OF_CPU_#";
			cpuLoadValue += m_util.intToString(i);
			cpuLoadValue += ":"; //format the results
			if (total[i] != oldtotal[i])
			{
				cpuLoadValue += m_util.intToString(100 * (load[i] - oldload[i]) / (total[i] - oldtotal[i]));

			}
			else
			{
				cpuLoadValue += m_util.intToString(0);
			}
			cpuLoadValue += ")";

			//set old values
			oldload[i] = load[i];
			oldtotal[i] = total[i];
		}
	}

	//invike explicit deletion of the arrays
	delete [] user;
	delete [] nice;
	delete [] sys;
	delete [] load;
	delete [] total;
	delete [] idle;
	delete [] frequency;
	return cpuLoadValue; //return the string with the cpu(s) values
}

/**
 * extract network flags (for libgtop) from a single long integer and translates
 * each to human-readable strings
 * @param netFlags is the long integer with libgtop's flags to be found
 * @return a formatted string with the all the found libgtop's flags
 * (in an human-readable format)
 */
const char* sysInfo::networkFlagToString(unsigned long netFlags)
{
	string convertedFlag; //return string

	//NOTE: the given long integer is OR'd (bit-wise operation) with each
	//libgtop flag constant to verify if the flag is present in the long integer.

	if(!(netFlags & GLIBTOP_IF_FLAGS_UP))
	{
		convertedFlag += "up,";
	}
	if(!(netFlags & GLIBTOP_IF_FLAGS_BROADCAST))
	{
		convertedFlag += "bcast,";
	}
	if(!(netFlags & GLIBTOP_IF_FLAGS_DEBUG))
	{
		convertedFlag += "dbg,";
	}
	if(!(netFlags & GLIBTOP_IF_FLAGS_LOOPBACK))
	{
		convertedFlag += "lo,";
	}
	if(!(netFlags & GLIBTOP_IF_FLAGS_POINTOPOINT))
	{
		convertedFlag += "ptp,";
	}
	if(!(netFlags & GLIBTOP_IF_FLAGS_RUNNING))
	{
		convertedFlag += "run,";
	}
	if(!(netFlags & GLIBTOP_IF_FLAGS_NOARP))
	{
		convertedFlag += "noarp,";
	}
	if(!(netFlags & GLIBTOP_IF_FLAGS_PROMISC))
	{
		convertedFlag += "promisc,";
	}
	if(!(netFlags & GLIBTOP_IF_FLAGS_ALLMULTI))
	{
		convertedFlag += "allmulti,";
	}
	if(!(netFlags & GLIBTOP_IF_FLAGS_OACTIVE))
	{
		convertedFlag += "oactive,";
	}
	if(!(netFlags & GLIBTOP_IF_FLAGS_SIMPLEX))
	{
		convertedFlag += "simplex,";
	}
	if(!(netFlags & GLIBTOP_IF_FLAGS_LINK0))
	{
		convertedFlag += "lnk0,";
	}
	if(!(netFlags & GLIBTOP_IF_FLAGS_LINK1))
	{
		convertedFlag += "lnk1,";
	}
	if(!(netFlags & GLIBTOP_IF_FLAGS_LINK2))
	{
		convertedFlag += "lnk2,";
	}
	if(!(netFlags & GLIBTOP_IF_FLAGS_ALTPHYS))
	{
		convertedFlag += "altphys,";
	}
	if(!(netFlags & GLIBTOP_IF_FLAGS_MULTICAST))
	{
		convertedFlag += "mcast,";
	}
	convertedFlag = convertedFlag.substr(0,convertedFlag.size()-1);

	//return a C-converted string with the network interface flags (in a human readable format)
	return convertedFlag.c_str();
}

/**
 * return all available informations about the status of a given network interface (i.e. eth0,lo,...)
 * @param iface is a C string with the name of the interface
 * @return a formatted string with the all info values for the given interface
 */
string sysInfo::getNetLoad(const char *iface)
{
	string netLoad; //return string
	char buffer[SI_MAX_BUFFER_SIZE]; //buffer used to store all the values
	glibtop_netload netload; //glibtop network load object

	//structs used to get the IP and the subnet mask associated to the given interface
	struct in_addr addr;
	struct in_addr subnet;

	//check if a valid interface name was entered => otherwise use "eth0"
	if(strlen(iface)==0)
	{
		return "<ERROR>: interface name required!";
	}

	//get network interface infos (via libgtop)
	glibtop_get_netload (&netload, iface);

	//conversions to get IP and subnet mask for the given interface
	addr.s_addr   = netload.address;
	subnet.s_addr = netload.subnet;
	char *strIP     = strdup(inet_ntoa(addr));
	char *strSubnet = strdup(inet_ntoa(subnet));

	//check if the IP and subnet mask were correctly filled
	if(strIP==NULL || strSubnet==NULL)
	{
		return "<ERROR>: internal error #1!";
	}

	//invoke networkFlagToString() to translate the unreadable integer )with the flags
	//for the given interface) to a human-redable format string
	const char* strFlags =  networkFlagToString((unsigned long)netload.if_flags);

	//fill the buffer with the infos for the given interface
	sprintf (buffer,
		"(NETWORK_LOAD:0x%08lx)(IFACE_FLAGS:%s)(ADDRESS:%s)(SUBNET:%s)(MTU:%ld)(COLLISIONS:%ld)(PACKETS_IN:%ld)(PACKETS_OUT:%ld)(PACKETS_TOTAL:%ld)(BYTES_IN:%ld)(BYTES_OUT:%ld)(BYTES_TOTAL:%ld)(ERRORS_IN:%ld)(ERRORS_OUT:%ld)(ERRORS_TOTAL:%ld)",
		(unsigned long) netload.flags,
		strFlags,
		strIP,
		strSubnet,
		(unsigned long) netload.mtu,
		(unsigned long) netload.collisions,
		(unsigned long) netload.packets_in,
		(unsigned long) netload.packets_out,
		(unsigned long) netload.packets_total,
		(unsigned long) netload.bytes_in,
		(unsigned long) netload.bytes_out,
		(unsigned long) netload.bytes_total,
		(unsigned long) netload.errors_in,
		(unsigned long) netload.errors_out,
		(unsigned long) netload.errors_total);
	netLoad += buffer; //append the buffer (C string) to the return (C++) string
	return netLoad; //return the string with all the network interface infos
}

/**
 * return a list of all mounted filesystems
 * @return a formatted string with the all the mounted filesystems
 */
string sysInfo::getMountList(string lineStart="", bool printBanner)
{
	string mountList=""; //return string
	glibtop_mountlist mountlist; //glibtop mount list object
	glibtop_mountentry *mountentries; //glibtop mount entry objects (as array of entries)
	char buffer[SI_MAX_BUFFER_SIZE]; //buffer used to store all the values

	//get mounted filesystems list (via libgtop)
	mountentries = glibtop_get_mountlist (&mountlist, 1);

	//check if the libgtop's mount entry object was correctly filled
	if (mountentries == NULL)
	{
		return "<ERROR>: internal error #2!";
	}

	if(printBanner==true)
	{
		mountList = "(MOUNT_LIST)";
	}

	//now we create a single formatted line with all the mounted filesystems
	for (unsigned int i=0; i<mountlist.number; i++)
	{
		sprintf (buffer,
			"(MOUNT_ENTRY:%s)(TYPE:%s)(DEVICE_NAME:%s)",
			mountentries [i].mountdir,
			mountentries [i].type,
			mountentries [i].devname);
		mountList += lineStart;
		mountList += buffer;
	}
	return mountList.substr(0,mountList.size()-2); //return the string with all the the mounted filesystems
}

/**
 * return a list of the usage usage of each mounted filesystems
 * @return a formatted string with the usage of all the mounted filesystems
 */
string sysInfo::getMountUsageList(string lineStart="", bool printBanner)
{
	string mountUsageList; //return string
	glibtop_fsusage fsusage; //glibtop filesystem usage object
	glibtop_mountlist mountlist; //glibtop mount list object
	glibtop_mountentry *mountentries; //glibtop mount entry objects (as array of entries)
	char buffer[SI_MAX_BUFFER_SIZE]; //buffer used to store all the values

	//get mounted filesystems list (via libgtop)
	mountentries = glibtop_get_mountlist (&mountlist, 1);

	//check if the libgtop's mount entry object was correctly filled
	if (mountentries == NULL)
	{
		return "<ERROR>: internal error #3!";
	}

	if(printBanner==true)
	{
		mountUsageList = "(MOUNT_USAGE_LIST)";
	}

	//now we create a single formatted line with the usage for each mounted filesystem
	for (unsigned int i=0; i<mountlist.number; i++)
	{
		//get the usage for a mounted filesystem
		glibtop_get_fsusage (&fsusage, mountentries[i].mountdir);

		sprintf (buffer,
			"(MOUNTDIR:%s)(BLKS:%ldK)(BFREE:%ldK)(BAVAIL:%ldK)(FILES:%ldK)(FFREE:%ldK)",
			mountentries [i].mountdir,
			(unsigned long)(fsusage.blocks/1000),
			(unsigned long)(fsusage.bfree/1000),
			(unsigned long)(fsusage.bavail/1000),
			(unsigned long)(fsusage.files/1000),
			(unsigned long)(fsusage.ffree)/1000);
		mountUsageList += lineStart;
		mountUsageList += buffer;
	}
	return mountUsageList.substr(0,mountUsageList.size()-2); //return the string with the usage of all the mounted filesystems
}



/**
 * return the usage of the swap
 * @return a formatted string with the swap usage
 */
string sysInfo::getSwapUsage()
{
	string 		swapUsage; //return string
	glibtop_swap	swapusage;
	char buffer[SI_MAX_BUFFER_SIZE]; //buffer used to store all the values

	//get swap usage
	glibtop_get_swap (&swapusage);

	//check if the libgtop's swap usage object was correctly filled
	if (&swapusage == NULL)
	{
		return "<ERROR>: internal error #5!";
	}

	sprintf (buffer,
		"(TOTAL:%ld)(USED:%ld)(FREE:%ld)(PAGES_IN:%ld)(PAGES_OUT:%ld)",
		(unsigned long)swapusage.total,
		(unsigned long)swapusage.used,
		(unsigned long)swapusage.free,
		(unsigned long)swapusage.pagein,
		(unsigned long)swapusage.pageout);

	swapUsage += buffer;
	return swapUsage;   //return the string with the swap usage
}

/**
 * return the usage of the swap
 * @return a formatted string with the swap usage
 */
string sysInfo::getLoadAverage()
{
	string 		loadAverage; //return string
	glibtop_loadavg	loadaverage;
	char buffer[SI_MAX_BUFFER_SIZE]; //buffer used to store all the values

	//get load average
	glibtop_get_loadavg (&loadaverage);

	//check if the libgtop's load average object was correctly filled
	if (&loadaverage == NULL)
	{
		return "<ERROR>: internal error #6!";
	}

	sprintf (buffer,
		"(LOAD_1MIN:%ge)(LOAD_5MINS:%ge)(LOAD_15MINS:%ge)(RUNNING_TASKS:%ld)(TOTAL_TASKS:%ld)(LAST_PID:%ld)",
		(double)loadaverage.loadavg[0],
		(double)loadaverage.loadavg[1],
		(double)loadaverage.loadavg[2],
		(unsigned long)loadaverage.nr_running,
		(unsigned long)loadaverage.nr_tasks,
		(unsigned long)loadaverage.last_pid);

	loadAverage += buffer;
	return loadAverage;   //return the string with the swap usage
}

/**
 * return the state of a given process
 * #param pid is the process identifier (PID) to look for
 * @return a formatted string state of a given process
 */
string sysInfo::getProcState(long pid)
{
	string 			procState; //return string
	glibtop_proc_state 	procstate;
	char buffer[SI_MAX_BUFFER_SIZE]; //buffer used to store all the values

	//get proc state
	glibtop_get_proc_state (&procstate, (pid_t)pid);

	//check if the libgtop's proc state object was correctly filled
	if (&procstate == NULL)
	{
		return "<ERROR>: internal error #7!";
	}

	if(strlen(procstate.cmd)==0)
	{
		return "<ERROR>: PID not exists!";
	}

	sprintf (buffer,
		"(COMMAND:%s)(STATUS:%c)(UID:%d)(GID:%d)",
		procstate.cmd,
		procstate.state,
		procstate.uid,
		procstate.gid);

	procState += buffer;
	return procState;   //return the string with the process state
}

/**
 * return the UIDs and the TTY status of a given process
 * #param pid is the process identifier (PID) to look for
 * @return a formatted string with the UIDs and the TTY status of a given process
 */
string sysInfo::getProcUIDAndTTY(long pid)
{
	string 			procUid; //return string
	glibtop_proc_uid 	procuid;
	char buffer[SI_MAX_BUFFER_SIZE]; //buffer used to store all the values

	//get load average
	glibtop_get_proc_uid (&procuid, (pid_t)pid);

	//check if the libgtop's proc UIDs and TTY status object was correctly filled
	if (&procuid == NULL)
	{
		return "<ERROR>: internal error #8!";
	}

	sprintf (buffer,
		"(UID:%ld)(EUID:%ld)(GID:%ld)(EGID:%ld)(PID:%ld)(PPID:%ld)(PGRP:%ld)(SESSIOND:%ld)(TTY:%ld)(TPGID:%ld)(PRIORITY:%ld)(NICE:%ld)",
		(unsigned long)procuid.uid,
		(unsigned long)procuid.euid,
		(unsigned long)procuid.gid,
		(unsigned long)procuid.egid,
		(unsigned long)procuid.pid,
		(unsigned long)procuid.ppid,
		(unsigned long)procuid.pgrp,
		(unsigned long)procuid.session,
		(unsigned long)procuid.tty,
		(unsigned long)procuid.tpgid,
		(unsigned long)procuid.priority,
		(unsigned long)procuid.nice);

	procUid = buffer;
	return procUid;   //return the string with the proc UIDs and TTY status
}

/**
 * return the memory status of a given process
 * #param pid is the process identifier (PID) to look for
 * @return a formatted string with the memory status of a given process
 */
string sysInfo::getProcMemory(long pid)
{
	string 			procMem; //return string
	glibtop_proc_mem 	procmem;
	char buffer[SI_MAX_BUFFER_SIZE]; //buffer used to store all the values

	//get load average
	glibtop_get_proc_mem (&procmem, (pid_t)pid);

	//check if the libgtop's proc memory status object was correctly filled
	if (&procmem == NULL)
	{
		return "<ERROR>: internal error #9!";
	}

	sprintf (buffer,
		"(SIZE:%ld)(VSIZE:%ld)(RESIDENT:%ld)(SHARE:%ld)(RSS:%ld)(RSS_RLIM:%ld)",
		(unsigned long)procmem.size,
		(unsigned long)procmem.vsize,
		(unsigned long)procmem.resident,
		(unsigned long)procmem.share,
		(unsigned long)procmem.rss,
		(unsigned long)procmem.rss_rlim);

	procMem = buffer;
	return procMem;   //return the string with the proc memory status
}

/**
 * return the proc time of a given process
 * #param pid is the process identifier (PID) to look for
 * @return a formatted string with the proc time of a given process
 */
string sysInfo::getProcTime(long pid)
{
	int 			numberOfCpus; //used to store the number of installed cpu
	string 			procTime; //return string
	glibtop_proc_time 	proctime;
	char buffer[SI_MAX_BUFFER_SIZE]; //buffer used to store all the values

	numberOfCpus = getCpusNumber(); //get the number of cpus for local node

	//get load average
	glibtop_get_proc_time (&proctime, (pid_t)pid);

	//check if the libgtop's proc proc time object was correctly filled
	if (&proctime == NULL)
	{
		return "<ERROR>: internal error #9!";
	}

	if (numberOfCpus==1) //there is only a single cpu
	{
		sprintf (buffer,
			"(CPU#:0)(START_TIME:%ld)(RTIME:%ld)(UTIME:%ld)(STIME:%ld)(CUTIME:%ld)(CSTIME:%ld)(TIMEOUT:%ld)(IT_REAL_VALUE:%ld)(FREQUENCY:%ld)",
			(unsigned long)proctime.start_time,
			(unsigned long)proctime.rtime,
			(unsigned long)proctime.utime,
			(unsigned long)proctime.stime,
			(unsigned long)proctime.cutime,
			(unsigned long)proctime.cstime,
			(unsigned long)proctime.timeout,
			(unsigned long)proctime.it_real_value,
			(unsigned long)proctime.frequency);
	}
	else
	{
		for(int i=0;i<numberOfCpus;i++) //for each installed cpu...
		{
			sprintf (buffer,
				"(CPU#:%d)(START_TIME:%ld)(RTIME:%ld)(UTIME:%ld)(STIME:%ld)(CUTIME:%ld)(CSTIME:%ld)(TIMEOUT:%ld)(IT_REAL_VALUE:%ld)(FREQUENCY:%ld)",
				i,
				(unsigned long)proctime.start_time,
				(unsigned long)proctime.rtime,
				(unsigned long)proctime.xcpu_utime[i],
				(unsigned long)proctime.xcpu_stime[i],
				(unsigned long)proctime.cutime,
				(unsigned long)proctime.cstime,
				(unsigned long)proctime.timeout,
				(unsigned long)proctime.it_real_value,
				(unsigned long)proctime.frequency);
		}
	}

	procTime = buffer;
	return procTime;   //return the string with the proc time status
}

/**
 * return the signal status of a given process
 * #param pid is the process identifier (PID) to look for
 * @return a formatted string with the signal status of a given process
 */
string sysInfo::getProcSignal(long pid)
{
	string 			procSignal; //return string
	glibtop_proc_signal 	procsignal;
	char buffer[SI_MAX_BUFFER_SIZE]; //buffer used to store all the values

	//get load average
	glibtop_get_proc_signal (&procsignal, (pid_t)pid);

	//check if the libgtop's proc signal status object was correctly filled
	if (&procsignal == NULL)
	{
		return "<ERROR>: internal error #9!";
	}

	sprintf (buffer,
		"(SIGNAL#0:%ld)(SIGNAL#1:%ld)(BLOCKED#0:%ld)(BLOCKED#1:%ld)(SIGIGNORE#0:%ld)(SIGIGNORE#1:%ld)(SIGCATCH#0:%ld)(SIGCATCH#1:%ld)",
		(unsigned long)procsignal.signal[0],
		(unsigned long)procsignal.signal[1],
		(unsigned long)procsignal.blocked[0],
		(unsigned long)procsignal.blocked[1],
		(unsigned long)procsignal.sigignore[0],
		(unsigned long)procsignal.sigignore[1],
		(unsigned long)procsignal.sigcatch[0],
		(unsigned long)procsignal.sigcatch[1]);

	procSignal = buffer;
	return procSignal;   //return the string with the proc signal status
}

