/***************************************************************************
*                          utils.cpp  -  description
*                             -------------------
*    begin                : Sam Feb  1 17:08:52 CET 2003
*    copyright            : (C) 2003 by the gomd group at savannah.org/
*    email                : rpons@rinu.org, brian@axehind.com, jp80@libero.it, mosixview@t-online.de
****************************************************************************/


/**
 * NOTES:
 * Created by rejected
 * Last update by rejected
 *
 * Contains the some useful functions used in the whole project
 */

#include "utils.h"
// helper functions

/**
 *  converts a string into an integer
 *  @param str is a string to be converted into an integer
 *  @return the string converted to integer
 */
int utils::stringToInt(string str)
{
	return(stringToInt(str.c_str()));
}

/**
 *  converts a string (const char*)  into an integer
 *  @param str is a string to be converted into an integer
 *  @return the string converted to integer
 */
int utils::stringToInt(const char* str)
{
	int ret;
	ret = atoi(str);
	if (errno==ERANGE)
	{
		ret = stringToInt(UTILS_CONV_ERROR);
	}
	return ret;
}

/**
 *  converts a string (const char*)  into an long
 *  @param str is a string to be converted into a long
 *  @return the string converted to long
 */
long utils::stringToLong(const char* str)
{
	long ret;
	ret = atol(str);
	if (errno==ERANGE)
	{
		ret = stringToInt(UTILS_CONV_ERROR);
	}
	return ret;
}

/**
 *  converts a string into a long
 *  @param str is a string to be converted into a long
 *  @return the string converted to long
 */
long utils::stringToLong(string str)
{
	return(stringToLong(str.c_str()));
}

/**
 *  converts an integer into a string
 *  @param i is an integer to be converted into an string
 *  @return a string including the input integer
 */
string utils::intToString(const int i)
{
	char buf[32];
	sprintf(buf,"%d",i);
	return string(buf);
}

/**
 *  converts a long into a string
 *  @param i is a long to be converted into an string
 *  @return a string including the input long
 */
string utils::longToString(const long i)
{
	char buf[32];
	sprintf(buf,"%ld",i);
	return string(buf);
}

/**
 *  converts a double into a string
 *  @param i is a double to be converted into an string
 *  @return a string including the input double
 */
string utils::doubleToString(const double i)
{
 	char buf[32];
	sprintf(buf,"%ge",i);
	return string(buf);
}

/**
 *  (re)initializes a string array
 *  @param array[] is the string array to be (re) initialize
 *  @param len is the number of the strings included into the string array
 */
void utils::initStringArray(string array[], int len)
{
	for(int i=0;i<len;i++)
	{
		array[i]="-1";
	}
}

/**
 *  trims a C++ string (removes starting and ending white spaces and tabs chars)
 *  @param toBeTrimmed is the reference to a string to be trimmed
 */
void utils::trim(string& toBeTrimmed)
{
	unsigned int i;

	if(toBeTrimmed=="" || toBeTrimmed.size()==0)  //if the string is empty...
	{
		return; //...return!
	}

	for(i=0;i<toBeTrimmed.size();i++)
	{
		if(toBeTrimmed.at(0)==' ' || toBeTrimmed.at(0)=='\t')  //if the string is empty...
		{
			toBeTrimmed.erase(0,1); //..remove it!
		}
		else
		{
			break;
		}
	}

	for(i=toBeTrimmed.size()-1;i>=0;i--)
	{
		if(toBeTrimmed.at(toBeTrimmed.size()-1)==' ' || toBeTrimmed.at(toBeTrimmed.size()-1)=='\t')  //if the string is empty...
		{
			toBeTrimmed.erase(toBeTrimmed.size()-1,1); //..remove it!
		}
		else
		{
			break;
		}
	}
}

/**
 * used to read a line form a file
 * @param inputFile is the input file
 * @return a string with the line read from the input file
 */
string utils::readLineFromFile(string inputFile)
{
	try
	{
		string ret = UTILS_IO_ERROR;
		char IOBuffer[101];
		ifstream ifile(inputFile.c_str()); //create input stream to read the file
		if (ifile.is_open()) //check if file is correctly open
		{
			ifile.getline(IOBuffer, 100); //read a line (max 100 chars) and put it in the buffer
			ret=""; //clear the return string
			ret.append(IOBuffer); //append the buffer
			ifile.close(); //close the stream
			return ret; //return the string
		}
		else
		{
			return UTILS_IO_ERROR;
		}
	}
	catch(...)
	{
		return UTILS_IO_ERROR;
	}
}


/**
 * used to search a substring into a file
 * @param inputFile is the input file
 * @param toSearch is the substring to be searched
 * @return the number of matches
 */
list<string> utils::searchSubstringInFile(string inputFile, string toSearch)
{
	string tmp;
	list<string> matchedLines;
	char c;

	tmp="";
	//cout << "<UT> opening file..." << endl;
	ifstream ifile(inputFile.c_str()); //create input stream to read the file
	if (ifile.is_open()) //check if file is correctly open
	{

		while(ifile.good())
		{
			c = ifile.get();
			if(c=='\n')
			{
				//cout << "<UT> getting line:" << tmp << endl;+
				if(tmp.find(toSearch)!=string::npos)
				{
					//cout << "<UT> matched line: " << tmp << endl;
					matchedLines.push_back(tmp);
				}
				tmp="";
			}
			else
			{
				tmp += c;
			}
		}

		ifile.close(); //close the stream
	}
	//cout << "<UT> finishing..." << endl;
	return matchedLines; //return the string
}

/**
 * used to search a substring into a file
 * @param inputFile is the input file
 * @param toSearch is the substring to be searched
 * @return the number of matches
 */
unsigned int utils::getSubStringOccurInFile(string inputFile, string toSearch)
{
	list<string> matchedLines = searchSubstringInFile(inputFile, toSearch);
	return matchedLines.size();
}


/**
 * used to read a line form a file
 * @param inputFile is the input file
 * @return a string with the line read from the input file
 */
string utils::readFile(string inputFile, string lineStart, string lineEnd)
{
	try
	{
		string ret=UTILS_IO_ERROR;
		char IOBuffer[101];
		ifstream ifile(inputFile.c_str()); //create input stream to read the file
		if (ifile.is_open()) //check if file is correctly open
		{
			ret="";

			while(! ifile.eof())
			{
				ifile.getline(IOBuffer, 100); //read a line (max 100 chars) and put it in the buffer
				if (IOBuffer[0] == 0) //end of special files (i.e. /proc/cpuinfo)
				{
					break;
				}
				else if (IOBuffer[0] == '#') //don't process lines starting with "#" (comments)
				{
					continue;
				}
				ret += lineStart;
				ret += IOBuffer; //append the buffer
				ret += lineEnd;
				trim(ret);
			}
			ifile.close(); //close the stream
			return ret; //return the string
		}
		else
		{
			return UTILS_IO_ERROR;
		}
	}
	catch(...)
	{
		return UTILS_IO_ERROR;
	}
}


/**
 * used to read a line form a file
 * @param inputFile is the input file
 * @return a string with the line read from the input file
 */
list<string> utils::readFileEx(string inputFile, string lineStart, string lineEnd)
{
	string tmp="";
	list<string> ret;
	char c;

	try
	{
		ifstream ifile(inputFile.c_str()); //create input stream to read the file
		if (ifile.is_open()) //check if file is correctly open
		{
			while(ifile.good())
			{
				c = ifile.get();
				if(c=='\n')
				{
					ret.push_back(tmp);
					tmp="";
				}
				else
				{
					tmp += c;
				}
			}

			ifile.close(); //close the stream

		}
		return ret; //return the string
	}
	catch(...)
	{
		return ret; //return the string
	}
}

/**
 * used to obtain a map of the pairs info,value included in a special file (like /proc/cpuinfo)
 * @param inputFile is the input file
 * @param strSeparator is the separator string between info and value
 * @return a map with the pairs info,value
 */
multimap<string,string> utils::parseSpecialFile(string inputFile, string strSeparator, int skipLines)
{
	char buffer[MAX_CHARS_TO_READ];
	string tmpLine;
	string tmpInfo;
	string tmpValue;
	multimap<string,string> mapOfFoundLines;
	ifstream ifile (inputFile.c_str());
	string::size_type separatorPos;

	if (! ifile.is_open())
	{
		cout << "[UTL]: error: opening file" << endl;
  		return mapOfFoundLines;
	}

	try
	{
		for(int i=0;i<skipLines;i++)
		{
			ifile.getline(buffer,MAX_CHARS_TO_READ);
		}

		while (! ifile.eof() )
		{
			ifile.getline(buffer,MAX_CHARS_TO_READ);
			//cout << "IFILE: " << buffer << endl;
			if (buffer[0] == 0) //end of special files (i.e. /proc/cpuinfo)
			{
				break;
			}
			else if (buffer[0] == '#') //don't process lines starting with "#" (comments)
			{
				continue;
			}

			tmpLine.erase();
			tmpLine.append(buffer);
			separatorPos = tmpLine.find(strSeparator);
			if(separatorPos!=string::npos)
			{
				tmpInfo = tmpLine.substr(0,separatorPos);
				tmpValue = tmpLine.substr(separatorPos+1);
				trim(tmpInfo);
				trim(tmpValue);
				mapOfFoundLines.insert(make_pair(tmpInfo,tmpValue));
			}
		}
		ifile.close();
		return mapOfFoundLines;
	}
	catch(...)
	{
		cout << "[UTL]: error: opening file" << endl;
		return mapOfFoundLines;
	}
}

/**
 * writes a value into a file
 * @param outputFile is the name of the output file
 * @param valueToWrite is the value to be written into the file
 * @return a boolean with the result of the operation
 */
bool utils::writeToFile(string outputFile, int valueToWrite)
{
	return writeToFile(outputFile, intToString(valueToWrite));
}

/**
 * writes a value into a file
 * @param outputFile is the name of the output file
 * @param valueToWrite is the value to be written into the file
 * @return a boolean with the result of the operation
 */
bool utils::writeToFile(string outputFile, string valueToWrite)
{
        ofstream ofile(outputFile.c_str()); //create output stream to write into the file
        if (ofile.is_open())
        {
                ofile << valueToWrite; //write a value to file (overwrite: no append mode!)
                ofile.close(); //close the stream
                return true;
        }
        else
        {
                return false;
        }
}

/**
 * writes a value into a file
 * @param outputFile is the name of the output file
 * @param valueToWrite is the value to be written into the file
 * @return a boolean with the result of the operation
 */
void utils::openFile(string outputFile, bool append=false)
{
	if(append==false)
	{
	        m_outputFile = new ofstream(outputFile.c_str()); //create output stream to write into the file
	}
	else
	{
	        m_outputFile = new ofstream(outputFile.c_str(), ios::out | ios::app); //create output stream to write into the file
	}
}

/**
 * writes a value into an open file
 * @param valueToWrite is the value to be written into the file
 * @return a boolean with the result of the operation
 */
bool utils::writeToOpenFile(string valueToWrite)
{
	if (m_outputFile->is_open())
        {
                *m_outputFile << valueToWrite; //write a value to file (overwrite: no append mode!)
                return true;
        }
        else
        {
                return false;
        }
}

/**
 * closes and open file
 * @return a boolean with the result of the operation
 */
bool utils::closeFile()
{
	if (m_outputFile->is_open()==true)
        {
        	m_outputFile->close(); //create output stream to write into the file
	        return true;
        }
        else
        {
                return false;
        }
}

/**
 * convert chars of a string to upper case
 * @param toBeConverted is the reference to the string to be converted
 */
void utils::ucase(string& toBeConverted)
{
	//this is the C++ STL way to converting all chars in a string to "upper" case
	transform (toBeConverted.begin(), toBeConverted.end(), toBeConverted.begin(), (int(*)(int)) toupper);
}

/**
 * convert chars of a string to lower case
 * @param toBeConverted is the reference to the string to be converted
 */
void utils::lcase(string& toBeConverted)
{
	//this is the C++ STL way to converting all chars in a string to "lower" case
	transform (toBeConverted.begin(), toBeConverted.end(), toBeConverted.begin(), (int(*)(int)) tolower);
}

/**
 * get the subnet from an IP
 * @param ipToParse is the IP to be parsed
 * @return the subnet of an IP
 */
string utils::getSubnetFromIP(string ipToParse)
{
	string::size_type pos;
	pos = ipToParse.rfind(".");
	if(pos==string::npos)
	{
		return UTILS_NET_ERROR;
	}
	return ipToParse.substr(0,pos);
}

/**
 * get the host number from an IP
 * @param ipToParse is the IP to be parsed
 * @return the host number of an IP
 */
string utils::getHostNumberFromIP(string ipToParse)
{
	string::size_type pos;
	pos = ipToParse.rfind(".");
	if(pos==string::npos)
	{
		return UTILS_NET_ERROR;
	}
	return ipToParse.substr(pos);
}

/**
 * verify if a string conatins letters
 * @toBeTested is the string to be tested
 * @retunr true if the string contains letters; false otherwise
 */
bool utils::containsLetters(string toBeTested)
{
	for(unsigned int i=0;i<toBeTested.size();i++)
	{
		if(isalpha(toBeTested.at(i)))
		{
			return true;
		}
	}
	return false;
}

//These two variables are used in execCommandAndGetChildOutput() class method
//These vars MUST be placed outside this function as they must be accessible
//from sigAlarmHandler() "old-C" function also. This function is the handler for SIGVALRM signal.
pid_t pid;                 //process id of the child proc created in execCommandAndGetChildOutput()
bool stop = false;  //when the handler sets this to true, the main loop in execCommandAndGetChildOutput() 
                                 //is forced to stop and the child-proc output is returned to the client

/**
 * handler for for SIGVALRM signal activated in execCommandAndGetChildOutput() function 
 * @param signo is the code of the received signal
 */
void sigAlarmHandler(int signo)
{
	cout << "[UTL]: process TTL expired => killing child process (pid=" << pid << ")" << endl;
	stop=true; //set the flag to true => we force the mail loop in execCommandAndGetChildOutput() to end
	kill(pid, SIGTERM); //kill the (child) process created in execCommandAndGetChildOutput()
}

/**
 * execute a command (in a separate child process) and return its output
 * @param cmdList is a list with the appname and the parameters already parsed
 * @param clientStream is the pointer to a TCPStream: if thsi pointer is valid, this function will return real-time output
 * @return the output of the executed command
 */
list<string> utils::execCommandAndReturnChildOutput(list<string> cmdList, TCPStream* clientStream)
{
	char fileChar;
	char** cmdAndArgs;
	char fileBuffer[SCX_MAX_CHARS_READ_FROM_FILE]; //buffer to store chars read from the pipe
	int i=0;
	int opipe[2];   //pipe descriptor (cells represent read and write streams for the pipe)
	list<string>::iterator iter;
	list<string> ret;
	string tmp;
	string completeLineBuffer;
	unsigned int cmdListSize;
	unsigned int readBytes;


	cmdListSize = cmdList.size();
	if(cmdListSize==0) //if the given list is empty...
	{
		return ret; //...exit!
	}

	//now we copy the list elements in an array
	cmdAndArgs = new char*[cmdListSize+1]; //the array conatins an extra cell...
	//for all the elements (command tokens) in the list...
	for(iter=cmdList.begin(); iter!=cmdList.end(); ++iter, i++) //for all the found words...
	{
		tmp = *iter;
		cmdAndArgs[i] = (char*)tmp.c_str(); //copy the list element to an array cell
	}
	//last element of the array must conatin the NULL-value => this is required by the execvp call!
	cmdAndArgs[i] = NULL;

	if(pipe(opipe)!=0) //if the pipe creation failed...
	{
		return ret; //...exit!
	}

	//if we specified a finite Time To Live...
	if(PROCESS_TIME_TO_LIVE!=INFINITE_TIME)
	{
		//...activate the timer: if the child process takes a time longer than PROCESS_TIME_TO_LIVE seconds
		//the child process will be automagically killed (and its output returned to the client).
		//Notice we use a special signal, SIGVTALRM: decrements only if the process is executing (we use
		//a VIRTUAL, process-based time, not the REAL, system-wide one! => hence we invoke
		//settimer() call with the ITIMER_VIRTUAL param)
		struct sigaction sa;
		struct itimerval timer;
		memset(&sa, 0, sizeof(sa));
		sa.sa_handler = &sigAlarmHandler; //set sigAlarmHandler() as the signal handler
		sa.sa_flags = SA_ONESHOT; //alarm will occur just once
		sigaction(SIGVTALRM, &sa, NULL); //make effective the signal-handler setting
		timer.it_value.tv_sec = PROCESS_TIME_TO_LIVE;	 //we'll wait for a standard time (properly a TTTL, "time to live"), defined in constants.h
		cout << "[UTL]: activating alarm for child-process (TTL=" << PROCESS_TIME_TO_LIVE << ")" << endl;
		setitimer (ITIMER_VIRTUAL, &timer, NULL); //activate the alarm
	}		
	else
	{
		//cout << "[UTL]: TTL set to \"infinite\". No alarm activated!" << endl;
	}	
	
	pid = fork();    //try to create a new child process
	if(pid==(pid_t)0) //child process code
	{
		close(opipe[0]);         //read  stream for the pipe is closed
		dup2(opipe[1],1);        //write stream for the pipe is open
		execvp(cmdAndArgs[0], cmdAndArgs);       //execute the command (with the args)

		//if we're here, a system-error occurred (usually the given program is unknown/unreachable)!!!
		cout << "[UTL]: error: cannot execute command (unknown/unreachable)!" << endl;
		exit(GOMD_EXIT_SUCCESS); //exit
	}
	else if(pid>0) //parent process code
	{
		close(opipe[1]);         //write  stream for the pipe is closed
		
		//repeat until child-process output (sent via pipe) ends
		//note: at each step we can read at max  SCX_MAX_CHARS_READ_FROM_FILE chars
		while((readBytes=(read(opipe[0],fileBuffer,SCX_MAX_CHARS_READ_FROM_FILE)))>0)
		{
			//now we search the buffer for "ccarriage return" and/ot "new line" symbols
			for(unsigned int i=0; i<readBytes; i++)
			{
				fileChar = fileBuffer[i];
				if(fileChar == '\r' || fileChar == '\n') //if we find these symbols we found a "complete line"
				{
					
					if(clientStream==NULL) //user asks standard (non real-time) output
					{
						ret.push_back(completeLineBuffer); //append the found "complete line" to the return string list
					}
					else //the user asks for real-time output
					{
						*clientStream << completeLineBuffer << endl;
						usleep(1);
					}
					completeLineBuffer=""; //clean the buffer
				}
				else
				{
					completeLineBuffer += fileChar; //if we're here we didn't find a "complete line" so we add the new char to the buffer
				}
			}
			if(completeLineBuffer.size()!=0) //if we're here we have still something in the buffer but we didn't find "\r" or "\n" symbols
			{
				if(clientStream==NULL) //user asks standard (non real-time) output
				{
					ret.push_back(completeLineBuffer); //append remaining buffer chars to the return string list
				}
				else //the user asks for real-time output
				{
					*clientStream << "[RT]:" << completeLineBuffer << endl;
					usleep(1);
				}
				completeLineBuffer="";
			}
			
			if(stop==true)
			{
				//if we're here, the child-process TTL expired (SIGVTALRM signal was raised)
				// so the child process was killed by sigAlarmHandler() signal handler and
				//we can return the child-process output we collected 'till the termination
				return ret;
			}
		}
		close(opipe[0]);
		waitpid(pid, NULL, 0);
	}
	else //fork failure handling
	{
		cout << "[UTL]: error: fork() failure! Alarm ignored.";
		return ret;
	}
	//cout << "[UTL]: Returning child-process output to the client" << endl;
	//cout << "           (no alarm was raised as the child-proc TTL didn't expire." << endl; 
	return ret;
}



/**
 * used to parse a command (split the command in tokens)
 * @param toBeParsed is the command to be parsed
 * @return an STL C++ list with the tokens found in the command
 */
list<string> utils::parseCommand(string toBeParsed)
{
	string tmp;
	list<string> ret;
	//cout << "[UTL]: string to be parsed: " << toBeParsed << endl;
	StringTokenizer st(toBeParsed.c_str()," ");
	StringTokenizer::iterator i;

	if(toBeParsed=="")
	{
		return ret;
	}

	for(i=st.begin();i!=st.end();++i) //for all the found words...
	{
		tmp = *i;
		if(tmp!="")
		{
			ret.push_back(tmp);
		}
	}
	return ret;
}

/**
 * used to get the file pointed by a symbolic link
 * @param linkFilePath is the path of the symbolic link
 * @return the path of the file pointed by the given symbolic link
 */
string utils::getFilenamePointedByLink(string linkFilePath)
{
	char targetFile[256];
	int r = readlink(linkFilePath.c_str(), targetFile, sizeof(targetFile));
	if(r==-1) //if something went wrong...
	{
		return UTILS_IO_ERROR; //...return an error!
	}
	else //...else
	{
		targetFile[r] = '\0';
		return string(targetFile); //return the found file
	}
}

/**
 * used to print the contents of a (string) linked list
 * @param ll is the input string linked list
 * @param prompt is a kind of title shown before the list contants
 * @param lineStart is a string placed before each linked list element
 */
void utils::printLinkedList(list<string> ll, string prompt, string lineStart)
{
	list<string>::iterator iter;
	cout << prompt << endl;
	
	if(&ll==NULL && ll.size()==0)
	{
		cout << "[UTL]: warning: linked list is empty!" << endl;
		return;
	}
	
	for(iter=ll.begin(); iter!=ll.end(); ++iter)
	{
		cout << lineStart << *iter << endl;
	}
}

/**
 * used to print the contents of a (string,string) multimap
 * @param ll is the input string multimap
 * @param prompt is a kind of title shown before the multimap contents
 * @param lineStart is a string placed before each multimap element
 */
void utils::printMultiMap(multimap<string,string> mm, string prompt, string lineStart)
{
	multimap<string,string>::iterator iter;
	cout << prompt << endl;
	
	if(&mm==NULL && mm.size()==0)
	{
		cout << "[UTL]: warning: multimap is empty!" << endl;
		return;
	}
	
	for(iter=mm.begin(); iter!=mm.end(); ++iter)
	{
		cout << lineStart << iter->first << ":" << iter->second << endl;
	}
}

/**
 * used to get a clone of a linked list of strings
 * @param sourceLL is an alias to the linked list to be cloned
 * @param destinationLL is an alias to the clone linked list
 */
void utils::cloneStringLinkedLists(list<string>& sourceLL, list<string>& destinationLL)
{
	list<string>::iterator iter;

	for(iter=sourceLL.begin(); iter!=sourceLL.end(); ++iter)
	{
		destinationLL.push_back(*iter);
	}
}

/**
 * used to get the local time (ISO 8601 standard-compliant time)
 * @return a string with the local time
 */
string utils::getLocalTime()
{
	char currTime[256]; //buffer used to contain the time
	time_t td;

	struct tm *dcp;
	time(&td);
	dcp = localtime(&td); //get the local time
	//get current time and format it as required by ISO 8601 standard
	strftime(currTime, 256, "_%Y%m%d_%H%M%S", dcp);
	return string(currTime);
}

/**
 * converts a boolean value to a "coloured" string
 * @param value is the boolean value to be converted
 * @return a "coloured" string obtained from the input boolean value
 */
string utils::boolToColorString(bool value)
{
	string ret;
	if(value==true)
	{
		ret.append(GREEN);
		ret.append("yes");
		ret.append(NORMAL);
	}
	else
	{
		ret.append(RED);
		ret.append("no");
		ret.append(NORMAL);
	}
	return ret;
}


